import timeout from 'connect-timeout'
import express from 'express'
import cors from 'cors'
import methods from './libs/methods.js'
import scheduler from './libs/scheduler.js'
import notifier from './libs/notifier.js'
import logger from './libs/logger.js'

const app = express()
const port = '1812'

app.use(cors())

let handleRequest = (req, res) => {
    let body = ''
    req.on('data', chunk => {
        // convert Buffer to string
        body += (null || chunk).toString()
    })
    req.on('end', async () => {
        let curMethod = methods[(req.originalUrl).replaceAll('/', '')]
        let result = curMethod ? await curMethod(JSON.parse(body)) : 'Método inválido'
        let resultCode = curMethod ? 200 : 404

        res.status(resultCode)
        res.json({
            'sucesso': !!curMethod,
            result
        })
        res.end()
    })
}

app.get('*', handleRequest)
app.post('*', handleRequest)
app.listen(port, '0.0.0.0', () => {
    logger.log(`\n`)
    logger.log('Server Started...')
    logger.log(`Listening on port ${port}`)
    logger.log(`\n`)
    notifier.sendNotification('Iniciando re-cacheamento do cardápio.', 'OdhenTAA - Serviço cacheamento de cardápio')

    methods.purgecache()
    methods.cacheallsst()
    scheduler.startSchedule(methods.cacheallsst)
})
app.use(timeout(9900000))
