import Util from '@/assets/libs/util'

let cart = []
let pagerCode = null
let pedidoComanda = null
let bandeiraCartao = null
let comandasAgregadas = []

const getProductPrice = function (product) {
    if ((Util.isTapMode() || Util.isBalancaMode()) && product.itemValue) {
        return product.itemValue
    } else if (Util.isBalancaMode()) {
        return Math.floor(product.quantity * product.totalItemPrice * 100) / 100
    } else {
        return Math.round(product.quantity * product.totalItemPrice * 100) / 100
    }
}

export default {
    setComanda: (_pedidoComanda) => {
        if (comandasAgregadas.length === 0) {
            pedidoComanda = _pedidoComanda
        }
        comandasAgregadas.push(_pedidoComanda)
    },
    getComanda: () => pedidoComanda,

    getComandasAgregadas: function () {
        return comandasAgregadas
    },

    getMainComandaAgregada: function () {
        if (comandasAgregadas && comandasAgregadas[0]) {
            return comandasAgregadas[0]
        }
        return {}
    },

    filterComanda: function (saleCode) {
        comandasAgregadas = comandasAgregadas.filter(ca => ca.saleCode !== saleCode)
        cart = cart.filter(ct => ct.comandaNumber !== saleCode)
    },

    getCart: function () {
        return cart
    },

    overrideCart: function (newCart) {
        cart = newCart
    },

    getPagerCode: function () {
        return pagerCode
    },

    setPagerCode: function (code) {
        pagerCode = code
        console.log(`pagerCode inserido: ${pagerCode}`)
    },

    getbandeiraCartao: function () {
        return bandeiraCartao
    },

    setbandeiraCartao: function (code) {
        bandeiraCartao = code
    },

    clearbandeiraCartao: function () {
        bandeiraCartao = null
        console.log(`Bandeira do Cartão Apagada`)
    },

    isCartEmpty: function () {
        return cart.length === 0
    },

    calculateItemPrice: function (product) {
        let value = getProductPrice(product)
        return Util.formatMoney(value)
    },

    // Retorna uma String com o valor total do carrinho, formatada no formato de dinheiro
    calculateTotalPrice: function (noServiceTax = false) {
        let totalPrice = 0
        cart.forEach((product) => {
            totalPrice += getProductPrice(product)
        })
        if (comandasAgregadas && comandasAgregadas[0] && comandasAgregadas[0].serviceTax !== null && noServiceTax === false) {
            let serviceTax = comandasAgregadas.map(ca => ca.serviceTax)
            serviceTax = serviceTax.reduce((a, b) => parseFloat(a || '0') + parseFloat(b || '0'), 0)
            if (serviceTax) {
                return `${Util.formatMoney(parseFloat(totalPrice) + parseFloat(serviceTax))}`
            }
        }
        return Util.formatMoney(totalPrice)
    },

    getTotalPriceAsFloat: function () {
        let cartTotalString = this.calculateTotalPrice()
        let cartTotal = cartTotalString.replace('R$', '').replace(',', '').replace('.', '').trim()
        return parseFloat(cartTotal) / 100
    },

    decreaseQtty: function (key, self) {
        if (cart[key].quantity > 1) {
            cart[key].quantity--
        } else {
            this.deleteProduct(key, self)
        }
    },

    increaseQtty: function (key) {
        cart[key].quantity++
    },

    editProduct: function (product, key) {
        cart[key] = product
    },

    navigateToCheckout: function (router) {
        if (cart.length > 0) {
            console.log(cart)
            router.push('checkout')
        }
    },

    navigateToHome: function (router) {
        router.push('home')
    },

    navigateToPayment: function (router) {
        router.push('payment')
    },

    navigateToPager: function (router) {
        router.push('pager')
    },

    fixProdForCart: function (prod) {
        if (!(prod.quantity && prod.unitValue && prod.totalItemPrice && prod.observations) && !prod.productCombo) {
            prod.quantity = prod.quantity || 1
            prod.unitValue = prod.price
            prod.totalItemPrice = prod.itemValue || Util.safeDecimalCalc(prod.unitValue, prod.discount, '-')
            prod.observations = prod.observations || []
            prod.productCombo = prod.productCombo || []
        }
        return prod
    },

    addToCart: function (prod) {
        cart.push(this.fixProdForCart(prod))
    },

    // Esta funcao é usada para limpeza de carrinho decorrente de acoes do usuario
    clearCart: function (self, route) {
        Util.showConfirmationDialog(self.$modal, () => {
            cart = []
            pagerCode = null
            self.$router.push(route)
        }, `Tem certeza que deseja limpar o carrinho? `)
    },

    // Esta funcao é usada para limpeza de carrinho decorrente de acoes do sistema
    resetCart: function () {
        cart = []
        pagerCode = null
        comandasAgregadas = []
    },

    clearCartAndNavigateHome (router) {
        this.resetCart()
        router.push('/')
    },

    deleteProduct: function (key, self) {
        Util.showConfirmationDialog(self.$modal, () => {
            cart.splice(key, 1)
            if (this.isCartEmpty()) {
                self.$router.push('Home')
            } else {
                self.$emit('reloadPrice')
            }
        }, `Tem certeza que deseja remover ${cart[key].productName} do carrinho? `)
    }
}
