import Util from '@/assets/libs/util'
import CartController from '@/assets/controllers/CartController'

// Utilizado para os pagamentos com crédito pessoal e débito consumidor
let consumerData = {
    comandaNumber: null,
    saleCode: null,
    clientCodeComanda: null,
    consumerCodeComanda: null,
    clientCode: null,
    consumerCode: null,
    logged: false,
    consumerName: '',
    consumersList: [],
    familyBalances: [],
    consumerDebit: {},
    personalCredit: {}
}

// Utilizado para os pagamentos com crédito pessoal e débito consumidor que possuem forma de pagamento complementar (cartão ou pix)
let compPaymentData = {
    principalPaymentMethod: null,
    compPaymentMethod: null,
    compValue: null,
    NSU: null,
    date: null,
    hour: null
}

const setCompPaymentData = (compPaymentMethod, compValue, NSU, date, hour) => {
    compPaymentData.compPaymentMethod = compPaymentMethod
    compPaymentData.compValue = compValue
    compPaymentData.NSU = NSU
    compPaymentData.date = date
    compPaymentData.hour = hour
}

const setPrincipalPaymentMethod = (principalPaymentMethod) => {
    compPaymentData.principalPaymentMethod = principalPaymentMethod
}

const getCompPaymentData = () => {
    return compPaymentData
}

const clearCompPaymentData = () => {
    compPaymentData.principalPaymentMethod = null
    compPaymentData.compPaymentMethod = null
    compPaymentData.compValue = null
    compPaymentData.NSU = null
    compPaymentData.date = null
    compPaymentData.hour = null
}

// Seta os dados do consumidor que fez login
const setConsumerData = (clientCode, consumerCode, consumerName = '', familyBalances, consumerDebit, personalCredit) => {
    consumerData.clientCode = clientCode
    consumerData.consumerCode = consumerCode
    consumerData.consumerName = consumerName
    consumerData.familyBalances = familyBalances
    consumerData.consumerDebit = consumerDebit
    consumerData.personalCredit = personalCredit
}

// Seta os dados do consumidor que estão na comanda, independente de estar logado no sistema ou não
const setConsumerDataComanda = (clientCodeComanda, consumerCodeComanda, comandaNumber, saleCode) => {
    consumerData.clientCodeComanda = clientCodeComanda
    consumerData.consumerCodeComanda = consumerCodeComanda
    consumerData.comandaNumber = comandaNumber
    consumerData.saleCode = saleCode
}

const setConsumersList = (consumersList) => {
    consumerData.consumersList = consumersList
}

const getConsumerData = () => {
    return consumerData
}

const clearConsumerData = () => {
    consumerData.comandaNumber = null
    consumerData.saleCode = null
    consumerData.clientCodeComanda = null
    consumerData.consumerCodeComanda = null
    consumerData.clientCode = null
    consumerData.consumerCode = null
    consumerData.logged = false
    consumerData.consumerName = ''
    consumerData.consumersList = []
    consumerData.familyBalances = []
    consumerData.consumerDebit = {}
    consumerData.personalCredit = {}
}

const userLogin = () => {
    consumerData.logged = true
}

const userIsLogged = () => {
    return consumerData.logged
}

const comandaIdentificada = () => {
    return consumerData.clientCodeComanda != null && consumerData.consumerCodeComanda != null
}

const treatFamilyBalances = (cart) => {
    let treatedFamilyBalances = []

    consumerData.personalCredit.familyBalances.forEach(family => {
        let familyBalance = family.balance
        let familyTotalCart = getFamilyTotalFromCart(family.familyCode, cart)
        let familyNextBalance = familyBalance - familyTotalCart

        treatedFamilyBalances.push({
            familyCode: family.familyCode,
            familyName: family.familyName,
            familyBalance: Util.formatMoney(familyBalance),
            familyTotalCart: Util.formatMoney(familyTotalCart),
            familyNextBalance: Util.formatMoney(familyNextBalance)
        })
    })

    return treatedFamilyBalances
}

const getFamilyTotalFromCart = (familyCode, cart) => {
    let familyTotal = 0
    let thisFamilyItems = cart.filter(prod => prod.familyCode === familyCode)

    thisFamilyItems.forEach(prod => {
        let price = prod.totalItemPrice
        // let quantity = prod.quantity ? prod.quantity : 1
        familyTotal += price // * quantity
    })

    return familyTotal
}

const getConsumerDebitBalance = () => {
    return consumerData.consumerDebit.balance
}

const canProceedToPayment = (method) => {
    let isPersonalCredit = method.IDTIPORECE === '9'
    let result
    if (isPersonalCredit) {
        result = checkPersonalCreditPayment()
    } else {
        result = checkConsumerDebitPayment()
    }
    return result
}

const checkPersonalCreditPayment = (method) => {
    let result = {
        canProceed: true,
        message: '',
        code: '000',
        showWarning: false,
        warningMessage: ''
    }

    let consumerData = getConsumerData()
    let cart = CartController.getCart()
    let cartTotal = CartController.getTotalPriceAsFloat()
    let allFamiliesSpentToday = 0
    let showWarningLowBalance = consumerData.personalCredit.showWarningLowBalance
    let lowBalanceFamilies = []
    let allFamiliesBalance = 0
    let allFamiliesLimit = 0

    let checkedConsumer = consumerData.clientCode === consumerData.clientCodeComanda && consumerData.consumerCode === consumerData.consumerCodeComanda

    if (!checkedConsumer) {
        result.canProceed = false
        result.message = 'O cliente que realizou login não é o mesmo identificado na comanda'
        result.code = '001'
        return result
    }

    for (let i = 0; i < consumerData.personalCredit.familyBalances.length; i++) {
        let currentFamily = consumerData.personalCredit.familyBalances[i]

        let familyMaxDailyValue = currentFamily.maxDailyValue
        let familySpentToday = currentFamily.spentToday
        let familyTotalCart = getFamilyTotalFromCart(currentFamily.familyCode, cart)

        allFamiliesSpentToday += familySpentToday

        if (familyMaxDailyValue > 0 && familySpentToday + familyTotalCart > familyMaxDailyValue) {
            result.canProceed = false
            result.message = `O valor máximo diário da família de produtos "${currentFamily.familyName}" foi excedido`
            result.code = '002'
            return result
        }

        let familyBalance = currentFamily.balance
        let negativeBalance = currentFamily.negativeBalance
        let methodBlockLowBalance = consumerData.personalCredit.blockLowBalance
        let balanceLimit = negativeBalance === 'S' ? methodBlockLowBalance : 0

        allFamiliesBalance += familyBalance
        allFamiliesLimit += balanceLimit

        if (familyBalance - familyTotalCart < balanceLimit) {
            result.canProceed = false
            result.message = `Seu saldo para a família de produtos "${currentFamily.familyName}" é insuficiente`
            result.code = '003'
            return result
        }

        if (showWarningLowBalance != null && familyBalance - familyTotalCart < showWarningLowBalance) {
            lowBalanceFamilies.push(currentFamily.familyName)
        }
    }

    let methodMaxDailyValue = consumerData.personalCredit.maxDailyValue

    if (methodMaxDailyValue > 0 && allFamiliesSpentToday + cartTotal > methodMaxDailyValue) {
        result.canProceed = false
        result.message = `O valor máximo diário desta forma de pagamento (${method.DSBUTTON}) foi excedido`
        result.code = '004'
        return result
    }

    if (allFamiliesBalance - cartTotal < allFamiliesLimit) {
        result.canProceed = false
        result.message = `Seu saldo é insuficiente para completar esta venda com a taxa de serviço`
        result.code = '005'
        return result
    }

    if (lowBalanceFamilies.length > 0) {
        result.showWarning = true
        result.warningMessage = buildWarningMessage(lowBalanceFamilies, showWarningLowBalance)
    }

    return result
}

const checkConsumerDebitPayment = (method) => {
    let result = {
        canProceed: true,
        message: '',
        showWarning: false,
        warningMessage: ''
    }

    let consumerData = getConsumerData()
    let cartTotal = CartController.getTotalPriceAsFloat()
    let checkedConsumer = consumerData.clientCode === consumerData.clientCodeComanda && consumerData.consumerCode === consumerData.consumerCodeComanda

    if (!checkedConsumer) {
        result.canProceed = false
        result.message = 'O cliente que realizou login não é o mesmo identificado na comanda'
        return result
    }

    let methodMaxDailyValue = consumerData.consumerDebit.maxDailyValue
    let spentToday = consumerData.consumerDebit.spentToday
    if (methodMaxDailyValue > 0 && spentToday + cartTotal > methodMaxDailyValue) {
        result.canProceed = false
        result.message = `O valor máximo diário desta forma de pagamento (${method.DSBUTTON}) foi excedido`
        return result
    }

    let balance = consumerData.consumerDebit.balance
    let blockLowBalance = consumerData.consumerDebit.blockLowBalance
    if (blockLowBalance != null && balance - cartTotal < blockLowBalance) {
        result.canProceed = false
        result.message = `Seu saldo é insuficiente para completar esta venda com a taxa de serviço`
        return result
    }

    let showWarningLowBalance = consumerData.consumerDebit.showWarningLowBalance
    if (showWarningLowBalance != null && balance - cartTotal < showWarningLowBalance) {
        result.showWarning = true
        result.warningMessage = `Seu saldo está abaixo de ${Util.formatMoney(showWarningLowBalance)}`
    }

    return result
}

const buildWarningMessage = (lowBalanceFamilies, showWarningLowBalance) => {
    let message = 'Seu saldo '

    if (lowBalanceFamilies.length === 1) {
        message += `da família de produtos "${lowBalanceFamilies[0]}"`
    } else {
        message += 'das famílias de produtos '
        lowBalanceFamilies.forEach((familyName, index) => {
            if (index === 0) {
                message += `"${familyName}"`
            } else if (index === lowBalanceFamilies.length - 1) {
                message += ` e "${familyName}"`
            } else {
                message += `, "${familyName}"`
            }
        })
    }

    message += ` está abaixo de ${Util.formatMoney(showWarningLowBalance)}`
    return message
}

const calculateExcessValue = (method) => {
    let isPersonalCredit = method.IDTIPORECE === '9'
    let excess
    if (isPersonalCredit) {
        excess = calculateExcessValueForPersonalCredit()
    } else {
        excess = calculateExcessValueForConsumerDebit()
    }
    return excess
}

const calculateExcessValueForPersonalCredit = () => {
    let excess = 0

    let consumerData = getConsumerData()
    let familyBalances = consumerData.personalCredit.familyBalances
    let cart = CartController.getCart()
    let cartTotal = CartController.getTotalPriceAsFloat()
    let allFamiliesSpentToday = 0

    familyBalances.forEach(family => {
        let familyExcess = 0
        let familyBalanceExcess = 0
        let familyMaxDailyExcess = 0

        let familyBalance = family.balance
        let negativeBalance = family.negativeBalance
        let methodBlockLowBalance = consumerData.personalCredit.blockLowBalance
        let balanceLimit = negativeBalance === 'S' ? methodBlockLowBalance : 0
        let familyMaxDailyValue = family.maxDailyValue
        let familySpentToday = family.spentToday
        let familyTotalCart = getFamilyTotalFromCart(family.familyCode, cart)
        allFamiliesSpentToday += familySpentToday

        if (familyBalance - familyTotalCart < balanceLimit) {
            familyBalanceExcess = balanceLimit - (familyBalance - familyTotalCart)
        }

        if (familyMaxDailyValue > 0 && familySpentToday + familyTotalCart > familyMaxDailyValue) {
            familyMaxDailyExcess = (familySpentToday + familyTotalCart) - familyMaxDailyValue
        }

        familyExcess = familyBalanceExcess > familyMaxDailyExcess ? familyBalanceExcess : familyMaxDailyExcess
        excess += familyExcess
    })

    let methodMaxDailyValue = consumerData.personalCredit.maxDailyValue
    let methodMaxDailyExcess = 0

    if (methodMaxDailyValue > 0 && allFamiliesSpentToday + cartTotal > methodMaxDailyValue) {
        methodMaxDailyExcess = (allFamiliesSpentToday + cartTotal) - methodMaxDailyValue
    }

    if (methodMaxDailyExcess > excess) excess = methodMaxDailyExcess

    let serviceTax = CartController.getMainComandaAgregada().serviceTax
    excess += serviceTax

    if (excess > cartTotal) excess = cartTotal

    return excess
}

const calculateExcessValueForConsumerDebit = () => {
    let excess = 0

    let consumerData = getConsumerData()
    let cartTotal = CartController.getTotalPriceAsFloat()

    let balance = consumerData.consumerDebit.balance
    let blockLowBalance = consumerData.consumerDebit.blockLowBalance
    if (blockLowBalance != null && balance - cartTotal < blockLowBalance) {
        excess = blockLowBalance - (balance - cartTotal)
    }

    let methodMaxDailyValue = consumerData.consumerDebit.maxDailyValue
    let spentToday = consumerData.consumerDebit.spentToday
    let methodMaxDailyExcess = 0
    if (methodMaxDailyValue > 0 && spentToday + cartTotal > methodMaxDailyValue) {
        methodMaxDailyExcess = (spentToday + cartTotal) - methodMaxDailyValue
    }

    if (methodMaxDailyExcess > excess) excess = methodMaxDailyExcess

    let serviceTax = CartController.getMainComandaAgregada().serviceTax
    excess += serviceTax

    if (excess > cartTotal) excess = cartTotal

    return excess
}

export default {
    setConsumerData,
    setConsumerDataComanda,
    setConsumersList,
    getConsumerData,
    clearConsumerData,
    userLogin,
    userIsLogged,
    comandaIdentificada,
    treatFamilyBalances,
    canProceedToPayment,
    calculateExcessValue,
    setCompPaymentData,
    setPrincipalPaymentMethod,
    getCompPaymentData,
    clearCompPaymentData,
    getConsumerDebitBalance
}
