/* eslint-disable no-undef */
/* eslint-disable no-unused-vars */
/* eslint-disable no-return-await */
import moment from 'moment'
import Util from '@/assets/libs/util'
import CartController from '@/assets/controllers/CartController'
import PrinterParameters from '@/assets/libs/PrinterParameters'
import requesModule from '@/assets/libs/requestModule'

const EVENT_PRINT = '/print'

// Busca as configurações parametrizadas para a impressora
const getPrinterInfo = () => {
    return {
        printerType: Util.getLocalConfigs('modeloImpressora'),
        kioskPort: Util.getLocalConfigs('kioskPort'),
        port: Util.getLocalConfigs('portaImpressora')
    }
}

// Envia um conjunto de comandos para a impressora realizar a impressao dos dados
const sendPrint = async (printerInfo, printerCommands, testURLPerifericos = null) => {
    printerCommands.forEach(pc => {
        console.log(pc && pc.parameters && pc.parameters.text)
    })
    return await requesModule.curlRequest(
        EVENT_PRINT,
        {
            printerInfo: printerInfo,
            commands: printerCommands
        },
        testURLPerifericos
    ).then(r => r)
}

// Cria uma linha tracejada respeitando a largura específica do papel da impressora em uso
const printLine = (printerParameters) => {
    let line = ''
    for (let i = 0; i < printerParameters.largura; i++) {
        line += '-'
    }
    line += printerParameters.comandoEnter
    return line
}

// Centraliza o texto de acordo com a largura específica do papel da impressora em uso
const centralizeText = (text, printerParameters) => {
    let textSize = text.length
    let lineSize = printerParameters.largura
    let leftSpaces = (lineSize - textSize) / 2
    let centralizedText = ''

    for (let i = 0; i < leftSpaces; i++) {
        centralizedText += ' '
    }
    centralizedText += text
    return centralizedText
}

// Formata o texto recebido, retornando a mesma String com as quebras de linha adequadas ao tamanho de papel da impressora
const addLineBreaks = (text, printerParameters, centralize = false) => {
    let words = text.split(' ')
    let returnStr = ''
    let currentLine = ''

    words.forEach((word, index) => {
        if ((currentLine + ' ' + word).length < printerParameters.largura) {
            if (index === 0) {
                currentLine += word
            } else {
                currentLine += ' ' + word
            }
        } else {
            if (centralize) {
                currentLine = centralizeText(currentLine, printerParameters)
            }
            currentLine += printerParameters.comandoEnter
            returnStr += currentLine
            currentLine = word
        }
    })

    if (centralize) {
        currentLine = centralizeText(currentLine, printerParameters)
    }
    returnStr += currentLine + printerParameters.comandoEnter
    return returnStr
}

const justifyText = (string1, string2, printerParameters) => {
    let textSize = (string1 + string2).length
    let blankSpaces = printerParameters.largura - textSize
    let returnStr = ''

    returnStr += string1
    for (let i = 0; i < blankSpaces; i++) {
        returnStr += ' '
    }
    returnStr += string2 + printerParameters.comandoEnter
    return returnStr
}

const formatCounter = (counter) => {
    if (counter < 10) {
        return '00' + counter
    } else if (counter < 100) {
        return '0' + counter
    } else {
        return '' + counter
    }
}

const formatBigNumber = (bigNumber) => {
    // Se este valor chegar do XML em formato de número e não de String, força a conversão
    bigNumber = '' + bigNumber

    let fragmentSize = 4
    let indexStart = 0
    let indexEnd = 4
    let fragment = ''
    let formattedBigNumber = ''
    do {
        fragment = bigNumber.substring(indexStart, indexEnd)
        formattedBigNumber += fragment + ' '
        indexStart += fragmentSize
        indexEnd += fragmentSize
    } while (indexStart < bigNumber.length)

    return formattedBigNumber
}

// Formata o número para ser exibido com duas casas decimais e utilizando "," para separar a parte inteira da decimal
const formatFloat = (value) => {
    if (typeof value !== 'string') {
        value = '' + value
    }
    let formattedFloat = parseFloat(value).toFixed(2)
    formattedFloat = formattedFloat.replace('.', ',')
    return formattedFloat
}

const formatTime = (time) => {
    if (typeof time !== 'string') {
        time = '' + time
    }

    let day = ''
    let month = ''
    let year = ''
    let hour = ''
    let minute = ''
    let second = ''

    if (time.length > 14) {
        day = time.substring(8, 10)
        month = time.substring(5, 7)
        year = time.substring(0, 4)
        hour = time.substring(11, 13)
        minute = time.substring(14, 16)
        second = time.substring(17, 19)
    } else {
        day = time.substring(6, 8)
        month = time.substring(4, 6)
        year = time.substring(0, 4)
        hour = time.substring(8, 10)
        minute = time.substring(10, 12)
        second = time.substring(12, 14)
    }

    let formattedTime = day + '/' + month + '/' + year + ' ' + hour + ':' + minute + ':' + second
    return formattedTime
}

const formatDocumentString = (docNumber) => {
    docNumber = '' + docNumber
    let formattedDocStr = ''

    if (docNumber.length === 11) {
        // Formata como CPF
        formattedDocStr += 'CPF CONSUMIDOR: '
        formattedDocStr += docNumber.substring(0, 3) + '.' + docNumber.substring(3, 6) + '.' +
        docNumber.substring(6, 9) + '-' + docNumber.substring(9, 11)
    } else {
        // Formata como CNPJ
        formattedDocStr += 'CNPJ CONSUMIDOR: '
        formattedDocStr += docNumber.substring(0, 2) + '.' + docNumber.substring(2, 5) + '.' +
        docNumber.substring(5, 8) + '/' + docNumber.substring(8, 12) + '-' + docNumber.substring(12, 14)
    }
    return formattedDocStr
}

const buildCouponHeader = (printerParameters) => {
    let text = ''
    text += printLine(printerParameters)
    text += centralizeText('PEDIDO', printerParameters) + printerParameters.comandoEnter
    text += centralizeText('DATA: ' + moment().format('DD/MM/YYYY HH:mm:ss'), printerParameters) + printerParameters.comandoEnter
    text += printLine(printerParameters)
    text += 'ORIGEM: TERMINAL DE AUTO ATENDIMENTO' + printerParameters.comandoEnter

    let nomeDoDispositivo = Util.getLocalConfigs('nomeDoDispositivo')
    text += addLineBreaks('GARCON: --  - ' + nomeDoDispositivo, printerParameters, false)

    text += printLine(printerParameters)
    text += printerParameters.comandoEnter
    return text
}

const buildCouponFooter = (printerParameters, saleData) => {
    let text = ''
    let senha = saleData.pagerId ? saleData.pagerId : ''
    text += printLine(printerParameters)

    if (senha !== '') {
        text += centralizeText('SENHA ' + senha, printerParameters) + printerParameters.comandoEnter
        text += printLine(printerParameters)
    }

    text += printerParameters.comandoEnter + printerParameters.comandoEnter + printerParameters.comandoEnter +
        printerParameters.comandoEnter + printerParameters.comandoEnter
    return text
}

const buildFiscalHeader = (printerParameters, nota) => {
    let data = nota.infNFe ? nota.infNFe : nota.infCFe
    let text = ''

    let address = ''
    address += data.emit.xNome
    address += ' CNPJ: ' + data.emit.CNPJ
    address += ' ' + data.emit.enderEmit.xLgr + ', '
    address += ' ' + data.emit.enderEmit.nro + ', '
    address += ' ' + data.emit.enderEmit.xBairro + ', '
    address += ' ' + data.emit.enderEmit.xMun
    let uf = data.emit.enderEmit.UF ? data.emit.enderEmit.UF : ''
    if (uf.length > 0) {
        address += ' - ' + uf
    }

    text += addLineBreaks(address, printerParameters, true)
    text += printLine(printerParameters)

    text += addLineBreaks('DOCUMENTO AUXILIAR DA NOTA FISCAL DE CONSUMIDOR ELETRONICA', printerParameters, true)
    text += printLine(printerParameters)
    text += addLineBreaks('Item | Codigo | Descricao | Qtde | UN | Vl.Unit | Vl.Tot', printerParameters, true)

    return text
}

const buildFiscalBody = (printerParameters, nota, protNfe, saleData, paymentMethod) => {
    let data = {}
    let infoSupl = {}
    if (nota.infNFe) {
        data = nota.infNFe
        infoSupl = nota.infNFeSupl
    } else {
        data = nota.infCFe
    }
    let text = ''
    let counter = 0
    let products = []

    // Se a lista de produtos da Nota Fiscal contiver apenas um item, ela não chegará como array, fazendo o forEach
    // quebrar. Esse bloco garante que os produtos cheguem ao forEach como um array
    let nfeProducts = data.det
    if (Array.isArray(nfeProducts)) {
        products = nfeProducts
    } else {
        products.push(nfeProducts)
    }

    products.forEach(product => {
        counter++
        let lineText = formatCounter(counter) + ' '
        lineText += product.prod.cProd + ' '
        lineText += product.prod.xProd + ' '
        lineText += formatFloat(product.prod.qCom) + ' '
        lineText += product.prod.uCom + ' '
        lineText += formatFloat(product.prod.vUnCom) + ' '
        lineText += formatFloat(product.prod.vProd) + ' '

        text += addLineBreaks(lineText, printerParameters) + printerParameters.comandoEnter
    })

    text += printLine(printerParameters)
    text += justifyText('QTDE. TOTAL DE ITENS ', counter, printerParameters)
    let formattedTotalValue = data.pag ? formatFloat(data.pag.detPag.vPag) : formatFloat(data.pgto.MP.vMP)

    let validationNumber = parseFloat(formattedTotalValue.replace(',', '.'))

    if (isNaN(validationNumber)) {
        try {
            let valuesArray = data.pag ? data.pag.detPag : data.pgto.MP
            let totalValue = 0
            if (valuesArray && valuesArray[0] && valuesArray[0].vPag) {
                valuesArray.forEach(element => {
                    totalValue = Math.round((parseFloat(totalValue) + parseFloat(element.vPag)) * 100) / 100
                })
            } else if (valuesArray && valuesArray[0] && valuesArray[0].vMP) {
                valuesArray.forEach(element => {
                    totalValue = Math.round((parseFloat(totalValue) + parseFloat(element.vMP)) * 100) / 100
                })
            }
            formattedTotalValue = formatFloat(totalValue)
        } catch (error) {
            let message = 'Houve um erro ao calcular os pagamentos recebidos no array durante a impressao da nota fiscal'
            console.log(message)
            Util.logComunication('PrinterController Exception ', {}, message + JSON.stringify(error))
        }
    }

    text += justifyText('VALOR TOTAL R$ ', formattedTotalValue, printerParameters)
    text += justifyText('VALOR A PAGAR R$ ', formattedTotalValue, printerParameters)
    text += justifyText('VALOR PAGO R$ ', formattedTotalValue, printerParameters)
    text += justifyText('FORMA PAGAMENTO ', paymentMethod.NMTIPORECE, printerParameters)

    // Esse bloco não é impresso nas notas de tipo SAT
    if (!nota.infCFe) {
        text += printLine(printerParameters)
        text += addLineBreaks('Consulte pela Chave de Acesso em', printerParameters, true)
        let infoSuplIsEmpty = Object.keys(infoSupl).length === 0
        let urlChave = !infoSuplIsEmpty ? infoSupl.urlChave : ''
        text += addLineBreaks(urlChave, printerParameters, true) + printerParameters.comandoEnter

        if (protNfe !== '') {
            let bigNumber = formatBigNumber(saleData.nfceAccessCode)
            text += addLineBreaks(bigNumber, printerParameters, true)
        }
    }

    text += printLine(printerParameters)

    let consumerCNPJ = ''
    let consumerCPF = ''

    try {
        consumerCNPJ = data.dest.CNPJ
    } catch (error) {
        console.log('Não foi possível ler o número do CNPJ a partir do XML da Nota Fiscal')
    }

    try {
        consumerCPF = data.dest.CPF
    } catch (error) {
        console.log('Não foi possível ler o número do CPF a partir do XML da Nota Fiscal')
    }

    if (consumerCNPJ) {
        let CNPJStr = formatDocumentString(consumerCNPJ, printerParameters)
        text += addLineBreaks(CNPJStr, printerParameters, true)
    } else if (consumerCPF) {
        let CPFStr = formatDocumentString(consumerCPF, printerParameters)
        text += addLineBreaks(CPFStr, printerParameters, true)
    } else {
        text += addLineBreaks('CONSUMIDOR NAO IDENTIFICADO', printerParameters, true)
    }

    text += printLine(printerParameters)

    // text += addLineBreaks('Numero ' + nfe.infNFe.ide.cNF + ' Serie ' + nfe.infNFe.ide.serie, printerParameters, true)
    let numeroNota = saleData.nfceCode
    text += addLineBreaks('Numero ' + numeroNota, printerParameters, true)
    let formattedDtEmi = data.ide.dhEmi ? formatTime(data.ide.dhEmi) : formatTime(data.ide.dEmi + data.ide.hEmi)
    text += addLineBreaks('Emissao ' + formattedDtEmi + ' - Via Consumidor', printerParameters, true) + printerParameters.comandoEnter

    if (protNfe !== '') {
        text += addLineBreaks('Protocolo de Autorizacao ' + protNfe.infProt.nProt, printerParameters, true)
        let formattedDtRecbto = formatTime(protNfe.infProt.dhRecbto)
        text += addLineBreaks('Data de Autorizacao ' + formattedDtRecbto, printerParameters, true)
    } else if (data.infAdic) {
        let info = data.infAdic.infCpl ? data.infAdic.infCpl : data.infAdic.obsFisco.xTexto
        text += addLineBreaks(info, printerParameters, true)
    }

    text += printLine(printerParameters) + printerParameters.comandoEnter

    return text
}

const buildFiscalFooter = (printerParameters, nota, saleData, paymentNumber) => {
    let data = nota.infNFe ? nota.infNFe : nota.infCFe
    let text = ''

    text += printerParameters.comandoEnter
    text += printLine(printerParameters)
    text += addLineBreaks('Informacao dos Tributos Totais Incidentes', printerParameters, true)

    let value = data.total.ICMSTot.vTotTrib ? data.total.ICMSTot.vTotTrib : data.total.vCFeLei12741
    let formattedValue = formatFloat(value)
    text += addLineBreaks('(Lei Federal 12.741/2012) R$ ' + formattedValue, printerParameters, true)
    text += printLine(printerParameters)

    text += addLineBreaks('Teknisa Software - www.teknisa.com', printerParameters, true)
    text += printLine(printerParameters)
    if (saleData.pagerId !== '') {
        text += centralizeText(`SENHA ${saleData.pagerId}`, printerParameters) + printerParameters.comandoEnter
        text += printLine(printerParameters)
    }
    if (paymentNumber !== '') {
        text += addLineBreaks(`NSU ${paymentNumber}`, printerParameters)
    }
    text += addLineBreaks('Filial: ' + saleData.subsidiaryCode + ' ' + saleData.subsidiaryName, printerParameters)
    text += addLineBreaks('Loja: ' + saleData.storeCode + ' ' + saleData.storeName, printerParameters)
    text += addLineBreaks('Caixa: ' + saleData.posCode + ' ' + saleData.posName, printerParameters)

    text += printerParameters.comandoEnter + printerParameters.comandoEnter + printerParameters.comandoEnter +
        printerParameters.comandoEnter + printerParameters.comandoEnter
    return text
}

const buildCouponCommands = (printerParameters, saleData) => {
    const commands = []
    try {
        const cart = CartController.getCart()
        let currentCommand

        // Header do cupom
        currentCommand = {
            name: 'text',
            parameters: {
                text: buildCouponHeader(printerParameters),
                italic: 0,
                underlined: 0,
                expanded: 0,
                bold: 0,
                letterType: printerParameters.tipoLetra
            }
        }
        commands.push(currentCommand)

        // Preenche os produtos no cupom
        cart.forEach(product => {
            let isComboProduct = product.productGroups && product.productGroups.length > 0
            let productPrint = product.productPrint

            if (!isComboProduct) {
                // Se for um produto simples, imprime
                let observations = []
                if (product.observations && product.observations.length > 0) {
                    observations = product.observations
                }
                currentCommand = buildProductLine(product.quantity, product.productName, printerParameters, observations)
                commands.push(currentCommand)
            } else if (isComboProduct && productPrint === 'S') {
                // Se for um produto combo e o produto principal é imprime sim
                // Imprime o produto principal
                let observations = []
                if (product.observations && product.observations.length > 0) {
                    observations = product.observations
                }
                currentCommand = buildProductLine(product.quantity, product.productName, printerParameters, observations)
                commands.push(currentCommand)

                // Imprime os filhos que forem imprime sim
                product.productCombo.forEach(childItem => {
                    if (childItem.groupPrint === 'S' || childItem.choosen) {
                        let firstLevelComboPrint = productPrint === 'S'
                        let groupPrint = childItem.groupPrint === 'S'
                        let isFosException = Util.isFosException(childItem, product.productCode, firstLevelComboPrint, groupPrint)

                        if (childItem.groupPrint === 'S' && !isFosException) {
                            let quantity = childItem.quantity ? childItem.quantity : 1
                            let childObservations = []
                            if (childItem.observations && childItem.observations.length > 0) {
                                childObservations = childItem.observations
                            }
                            currentCommand = buildProductLine(quantity, childItem.subItemName, printerParameters, childObservations)
                            commands.push(currentCommand)
                        }

                        // trata os filhos do combo do combo
                        if (childItem.choosen) {
                            let subSubGroups = childItem.choosen
                            let keys = Object.keys(subSubGroups)
                            keys.forEach(key => {
                                let currentGroup = subSubGroups[key]
                                currentGroup.forEach(subSubProd => {
                                    let childPrint = subSubProd.groupPrint === 'S'
                                    // Neste bloco de código o pai só pode ser imprime sim, porque já foi verificado acima.
                                    // Então basta verificar se o filho também é imprime sim
                                    if (childPrint) {
                                        let quantity = subSubProd.quantity ? subSubProd.quantity : 1
                                        let subSubProdObservations = []
                                        if (subSubProd.observations && subSubProd.observations.length > 0) {
                                            subSubProdObservations = subSubProd.observations
                                        }
                                        currentCommand = buildProductLine(quantity, subSubProd.subItemName, printerParameters, subSubProdObservations)
                                        commands.push(currentCommand)
                                    }
                                })
                            })
                        }
                    }
                })
            } else if (isComboProduct && productPrint === 'N') {
                // Se for um produto combo e o produto principal é imprime não.
                // Não imprime o principal e imprime todos os filhos
                // (não importa se os filhos são imprime SIM ou imprime NÃO)
                product.productCombo.forEach(childItem => {
                    let quantity = childItem.quantity ? childItem.quantity : 1
                    let childObservations = []
                    if (childItem.observations && childItem.observations.length > 0) {
                        childObservations = childItem.observations
                    }
                    currentCommand = buildProductLine(quantity, childItem.subItemName, printerParameters, childObservations)
                    commands.push(currentCommand)

                    // trata os filhos do combo do combo
                    if (childItem.choosen) {
                        let subSubGroups = childItem.choosen
                        let keys = Object.keys(subSubGroups)
                        keys.forEach(key => {
                            let currentGroup = subSubGroups[key]
                            currentGroup.forEach(subSubProd => {
                                // Imprime todos os itens filhos de combo do combo, sejam imprime sim ou imprime nao
                                let quantity = subSubProd.quantity ? subSubProd.quantity : 1
                                let subSubProdObservations = []
                                if (subSubProd.observations && subSubProd.observations.length > 0) {
                                    subSubProdObservations = subSubProd.observations
                                }
                                currentCommand = buildProductLine(quantity, subSubProd.subItemName, printerParameters, subSubProdObservations)
                                commands.push(currentCommand)
                            })
                        })
                    }
                })
            }
        })

        // Footer do cupom
        currentCommand = {
            name: 'text',
            parameters: {
                text: buildCouponFooter(printerParameters, saleData),
                italic: 0,
                underlined: 0,
                expanded: 0,
                bold: 0,
                letterType: printerParameters.tipoLetra
            }
        }
        commands.push(currentCommand)

        // Corta o papel na impressora
        currentCommand = {
            name: 'cutPaper',
            parameters: []
        }

        commands.push(currentCommand)
    } catch (error) {
        console.log(error.message)
    }
    return commands
}

const buildTestPrinterCommands = (printerParameters) => {
    const commands = []

    let text = ''
    text += printLine(printerParameters)
    text += centralizeText('TESTE DE IMPRESSORA', printerParameters) + printerParameters.comandoEnter
    text += printLine(printerParameters)
    text += addLineBreaks('Teste de impressao executado com sucesso em ' + moment().format('DD/MM/YYYY HH:mm:ss'), printerParameters) + printerParameters.comandoEnter
    text += addLineBreaks('Origem: Terminal de Auto Atendimento', printerParameters, false)
    text += printLine(printerParameters) + printerParameters.comandoEnter + printerParameters.comandoEnter

    commands.push({
        name: 'text',
        parameters: {
            text: text,
            italic: 0,
            underlined: 0,
            expanded: 0,
            bold: 0,
            letterType: printerParameters.tipoLetra
        }
    })

    // Corta o papel na impressora
    let currentCommand = {
        name: 'cutPaper',
        parameters: []
    }
    commands.push(currentCommand)

    return commands
}

const buildProductLine = (quantity, productName, printerParameters, observations = []) => {
    let text = ''
    let productLine = '---> ' + ' ' + quantity + ' ' + productName.toUpperCase() + printerParameters.comandoEnter
    text += addLineBreaks(productLine, printerParameters)

    observations.forEach(obs => {
        if (obs.price && obs.price > 0) {
            let obsName = obs.obsName ? obs.obsName.toUpperCase() : obs.subItemObsName.toUpperCase()
            let obsLine = '---> ' + ' ' + obsName + printerParameters.comandoEnter
            text += addLineBreaks(obsLine, printerParameters)
        }
    })

    return {
        name: 'text',
        parameters: {
            text: text,
            italic: 0,
            underlined: 0,
            expanded: 0,
            bold: 0,
            letterType: printerParameters.tipoLetra
        }
    }
}

const buildFiscalCommands = (printerParameters, nota, protNfe, saleData, paymentMethod, paymentNumber) => {
    const commands = []
    let currentCommand

    // Header da Nota Fiscal
    currentCommand = {
        name: 'text',
        parameters: {
            text: buildFiscalHeader(printerParameters, nota),
            italic: 0,
            underlined: 0,
            expanded: 0,
            bold: 0,
            letterType: printerParameters.tipoLetra
        }
    }
    commands.push(currentCommand)

    // Preenche os produtos na Nota Fiscal
    currentCommand = {
        name: 'text',
        parameters: {
            text: buildFiscalBody(printerParameters, nota, protNfe, saleData, paymentMethod),
            italic: 0,
            underlined: 0,
            expanded: 0,
            bold: 0,
            letterType: printerParameters.tipoLetra
        }
    }
    commands.push(currentCommand)

    // Esse bloco só imprime nas notas do tipo SAT
    if (nota.infCFe) {
        let keyNumber = saleData.nfceAccessCode
        if (keyNumber) {
            keyNumber = keyNumber.replace('CFe', '')
        }
        let bigNumber = formatBigNumber(keyNumber)

        let text = addLineBreaks(bigNumber, printerParameters, true) + printerParameters.comandoEnter
        currentCommand = {
            name: 'text',
            parameters: {
                text: text,
                italic: 0,
                underlined: 0,
                expanded: 0,
                bold: 0,
                letterType: printerParameters.tipoLetra
            }
        }
        commands.push(currentCommand)

        let barCode1 = ''
        barCode1 = keyNumber.substring(0, 22)

        let barCode2 = ''
        barCode2 = keyNumber.substring(22, 45)

        currentCommand = {
            name: 'barCode',
            parameters: {
                text: barCode1,
                height: 40,
                width: 0,
                position: 0,
                font: 0,
                margin: 25
            }
        }
        commands.push(currentCommand)

        currentCommand = {
            name: 'text',
            parameters: {
                text: printerParameters.comandoEnter,
                italic: 0,
                underlined: 0,
                expanded: 0,
                bold: 0,
                letterType: printerParameters.tipoLetra
            }
        }
        commands.push(currentCommand)

        currentCommand = {
            name: 'barCode',
            parameters: {
                text: barCode2,
                height: 40,
                width: 0,
                position: 0,
                font: 0,
                margin: 25
            }
        }
        commands.push(currentCommand)

        currentCommand = {
            name: 'text',
            parameters: {
                text: printerParameters.comandoEnter,
                italic: 0,
                underlined: 0,
                expanded: 0,
                bold: 0,
                letterType: printerParameters.tipoLetra
            }
        }
        commands.push(currentCommand)
    }

    // Gera o QRCode na Nota Fiscal
    currentCommand = {
        name: 'qrCode',
        parameters: {
            text: saleData.qrCode,
            italic: 0,
            underlined: 0,
            expanded: 0,
            bold: 0,
            letterType: printerParameters.tipoLetra
        }
    }
    commands.push(currentCommand)

    // Footer da Nota Fiscal
    currentCommand = {
        name: 'text',
        parameters: {
            text: buildFiscalFooter(printerParameters, nota, saleData, paymentNumber),
            italic: 0,
            underlined: 0,
            expanded: 0,
            bold: 0,
            letterType: printerParameters.tipoLetra
        }
    }
    commands.push(currentCommand)

    // Corta o papel na impressora
    currentCommand = {
        name: 'cutPaper',
        parameters: []
    }
    commands.push(currentCommand)
    return commands
}

/**
 * Insere em cada produto do carrinho o código das impressoras de produção (impressoras de cozinha) parametrizadas para
 * aquele produto e retorna um array contendo todas as impressoras de produção que serão utilizadas para imprimir este
 * pedido na cozinha
 */
const handleKitchenPrinters = (cart, productsPrinters) => {
    let kitchenPrinters = []

    cart.forEach(product => {
        // Trata das impressoras de produção (impressoras da cozinha) do produto pai
        let thisProdPrinters = productsPrinters.find(element => element.productCode === product.productCode)
        if (thisProdPrinters !== undefined) {
            product.seqPrinterProduct = thisProdPrinters.seqPrinterProduct
            product.seqPrinterProduct2 = thisProdPrinters.seqPrinterProduct2
            product.seqPrinterPuxa = thisProdPrinters.seqPrinterPuxa
            if (!kitchenPrinters.includes(product.seqPrinterProduct) && product.seqPrinterProduct !== null) {
                kitchenPrinters.push(product.seqPrinterProduct)
            }
            if (!kitchenPrinters.includes(product.seqPrinterProduct2) && product.seqPrinterProduct2 !== null) {
                kitchenPrinters.push(product.seqPrinterProduct2)
            }
            if (!kitchenPrinters.includes(product.seqPrinterPuxa) && product.seqPrinterPuxa !== null) {
                kitchenPrinters.push(product.seqPrinterPuxa)
            }
        }

        // Trata das impressoras de produção (impressoras da cozinha) dos produtos filhos, se houver
        if (product.productGroups.length > 0) {
            product.productCombo.forEach(childItem => {
                let thisChildProdPrinters = productsPrinters.find(element => element.productCode === childItem.subItemCode)
                if (thisChildProdPrinters !== undefined) {
                    childItem.seqPrinterProduct = thisChildProdPrinters.seqPrinterProduct
                    childItem.seqPrinterProduct2 = thisChildProdPrinters.seqPrinterProduct2
                    childItem.seqPrinterPuxa = thisChildProdPrinters.seqPrinterPuxa
                    if (!kitchenPrinters.includes(childItem.seqPrinterProduct) && childItem.seqPrinterProduct !== null) {
                        kitchenPrinters.push(childItem.seqPrinterProduct)
                    }
                    if (!kitchenPrinters.includes(childItem.seqPrinterProduct2) && childItem.seqPrinterProduct2 !== null) {
                        kitchenPrinters.push(childItem.seqPrinterProduct2)
                    }
                    if (!kitchenPrinters.includes(childItem.seqPrinterPuxa) && childItem.seqPrinterPuxa !== null) {
                        kitchenPrinters.push(childItem.seqPrinterPuxa)
                    }
                }
            })
        }
    })

    return kitchenPrinters
}

const buildCancelCommands = (printerParameters, cancelProof, cancelSaleResponse, nfceCode) => {
    // let externalLibs = require('./../../assets/externalLibs/x2js')
    // let x2js = new externalLibs.X2JS()
    // let xmlEnvio = x2js.xml_str2json(cancelSaleResponse.data.cancel.XMLENVIO)
    // let xmlRetorno = x2js.xml_str2json(cancelSaleResponse.data.cancel.XMLRETORNO)

    // let cnpj = ''
    // let date = ''
    // let accessKey = ''
    // let total = ''
    // let protocolNumber = null

    const commands = []
    let text = ''

    // try {
    //     cnpj = xmlEnvio.envEvento.evento.infEvento.CNPJ
    //     cnpj = formatDocumentString(cnpj)
    //     cnpj = cnpj.replace('CNPJ CONSUMIDOR: ', '')
    // } catch (error) {
    //     console.log('Houve um erro ao ler dados do XML de cancelamento (XML de envio)')
    //     console.log(error.message)
    // }

    // try {
    //     accessKey = xmlRetorno.Envelope.Body.nfeResultMsg.retEnvEvento.retEvento.infEvento.chNFe
    //     accessKey = formatBigNumber(accessKey)
    //     date = xmlRetorno.Envelope.Body.nfeResultMsg.retEnvEvento.retEvento.infEvento.dhRegEvento
    //     date = formatTime(date)
    // } catch (error) {
    //     console.log('Houve um erro ao ler dados do XML de cancelamento (XML de retorno)')
    //     console.log(error.message)
    // }

    // try {
    //     protocolNumber = xmlRetorno.Envelope.Body.nfeResultMsg.retEnvEvento.retEvento.infEvento.nProt
    // } catch (error) {
    //     console.log('O cancelamento foi realizado em contingência')
    // }

    // text += addLineBreaks('[NOME DO ESTABELECIMENTO]', printerParameters, true)
    // text += addLineBreaks(`CNPJ: ${cnpj}`, printerParameters, true)
    // text += addLineBreaks('[ENDEREÇO DO ESTABELECIMENTO]', printerParameters, true)

    text += printLine(printerParameters)

    // text += addLineBreaks('Nota Fiscal Eletronica para Consumidor', printerParameters, true)
    text += addLineBreaks('CANCELAMENTO DE VENDA', printerParameters, true) // + printerParameters.comandoEnter
    // text += addLineBreaks(`Numero: ${nfceCode}`, printerParameters, true)
    // text += addLineBreaks(`Data: ${date}`, printerParameters, true) + printerParameters.comandoEnter
    // text += addLineBreaks('CHAVE DE ACESSO', printerParameters, true)
    // text += addLineBreaks(accessKey, printerParameters, true)

    text += printLine(printerParameters)

    // text += addLineBreaks('[PRODUTOS]', printerParameters, true)
    // text += printLine(printerParameters)
    // text += justifyText('TOTAL:', total, printerParameters, true)

    // text += printLine(printerParameters)

    // if (protocolNumber) {
    //     text += addLineBreaks(`Protocolo de Autorizacao ${protocolNumber}`, printerParameters, true)
    // } else {
    //     text += addLineBreaks('Cancelamento realizado em contingência', printerParameters, true)
    //     text += addLineBreaks('Sem protocolo de autorização', printerParameters, true)
    // }

    // text += printLine(printerParameters)

    let cancelProofFormatted = cancelProof.replaceAll('\n', printerParameters.comandoEnter)

    commands.push(newTextCommand(text, printerParameters))
    commands.push(newTextCommand(cancelProofFormatted, printerParameters))
    commands.push(cutPaperCommand())

    return commands
}

const newTextCommand = (text, printerParameters) => {
    return {
        name: 'text',
        parameters: {
            text: text,
            italic: 0,
            underlined: 0,
            expanded: 0,
            bold: 0,
            letterType: printerParameters.tipoLetra
        }
    }
}

const cutPaperCommand = () => {
    return {
        name: 'cutPaper',
        parameters: []
    }
}

export default {
    simplePrint: async function (text) {
        let printerInfo = getPrinterInfo()
        let printerParameters = PrinterParameters.getPrinterParameters(printerInfo.printerType)

        let printerText = text.replaceAll('\n', printerParameters.comandoEnter)
        console.log(printerText)
        let printerCommands = [{
            name: 'text',
            parameters: {
                text: printerText,
                italic: 0,
                underlined: 0,
                expanded: 0,
                bold: 0,
                letterType: printerParameters.tipoLetra
            }
        }]

        // Corta o papel na impressora
        let currentCommand = {
            name: 'cutPaper',
            parameters: []
        }
        printerCommands.push(currentCommand)

        return await requesModule.curlRequest(
            EVENT_PRINT,
            {
                printerInfo: printerInfo,
                commands: printerCommands
            },
            Util.getLocalConfigs('urlPerifericos')
        ).then(r => r)
    },

    relatorioFechamentoDodia: async function (shouldPrint = true) {
        let fechamento = await requesModule.postRequest('pos/close', {...Util.getGeneralInfo()}, 'parent', (2 * 60 * 1000)).then(r => r && r.data)
        console.log(fechamento)
        if (fechamento.closing.error) {
            Util.showErrorNotification(fechamento.closing.message)
        } else if (shouldPrint) {
            this.simplePrint(fechamento.closing.data.dia)
        }
    },

    // Imprime folha de teste da impressora
    testPrinter: async function (testPrinterInfo, testURLPerifericos) {
        try {
            let printerParameters = PrinterParameters.getPrinterParameters(testPrinterInfo.printerType)
            let printerCommands = buildTestPrinterCommands(printerParameters)
            let result = await sendPrint(testPrinterInfo, printerCommands, testURLPerifericos)
            return result.data
        } catch (error) {
            return {
                error: true,
                message: error.message
            }
        }
    },

    // Imprime o cupom simples do consumidor com o pedido e numero da senha
    printCoupon: async function (saleData) {
        let printerInfo = getPrinterInfo()
        let printerParameters = PrinterParameters.getPrinterParameters(printerInfo.printerType)
        let printerCommands = buildCouponCommands(printerParameters, saleData)
        return await sendPrint(printerInfo, printerCommands)
    },

    // Imprime o Documento Auxiliar da Nota Fiscal Eletronica
    printFiscal: async function (fiscalXML, saleData, paymentMethod, paymentNumber) {
        let externalLibs = require('./../../assets/externalLibs/x2js')
        let x2js = new externalLibs.X2JS()
        let jsonFiscal = x2js.xml_str2json(fiscalXML)

        // Contem as informacoes da nota fiscal propriamente dita
        let nota = ''
        // Contem as informacoes do protocolo da nota fiscal
        let protNfe = ''
        // Se a nota fiscal foi gerada em contingencia, o XML nao traz os dados de protocolo da nota
        try {
            // Nota NFe recebida completa
            nota = jsonFiscal.nfeProc.NFe
            protNfe = jsonFiscal.nfeProc.protNFe
        } catch (error) {
            // Nota NFe recebida em contingência
            nota = jsonFiscal.NFe
        }

        if (!nota) {
            try {
                // Nota de tipo SAT
                nota = jsonFiscal.CFe
            } catch (error) {
                console.log('Houve um problema com o formato da nota fiscal recebida')
            }
        }

        let printerInfo = getPrinterInfo()
        let printerParameters = PrinterParameters.getPrinterParameters(printerInfo.printerType)
        let printerCommands = buildFiscalCommands(printerParameters, nota, protNfe, saleData, paymentMethod, paymentNumber)
        return await sendPrint(printerInfo, printerCommands)
    },

    // Imprime o Comprovante de cancelamento de TEF / Nota Fiscal
    printCancel: async function (cancelProof, cancelSaleResponse, nfceCode) {
        let printerInfo = getPrinterInfo()
        let printerParameters = PrinterParameters.getPrinterParameters(printerInfo.printerType)
        let printerCommands = buildCancelCommands(printerParameters, cancelProof, cancelSaleResponse, nfceCode)
        return await sendPrint(printerInfo, printerCommands)
    },

    // Imprime os cupons de produtos nas impressoras de produção respectivas
    sendKitchenPrints: async function () {
        let cart = CartController.getCart()
        let storePrinters = PrintersBySector.getStorePrinters()
        let productsPrinters = PrintersBySector.getProductsPrinters()
        let kitchenPrinters = handleKitchenPrinters(cart, productsPrinters)

        kitchenPrinters.forEach(async (printer) => {
            let printerInfo = getPrinterInfo() // Tem que ser ajustado para cada tipo de impressora (ainda não existe endpoint que traga o MODELO da impressora)
            let printerParameters = PrinterParameters.getPrinterParameters(printerInfo.printerType)
            let printerCommands = buildKitchenCommands(printerParameters, printer, cart)
            await sendPrint(printerInfo, printerCommands) // Ainda estamos mandando para a impressora padrao e nao para a impressora da cozinha selecionada
            // Tratar os erros, como é feito com as outras chamadas de impressão dentro de SaleController (porém, tratar aqui, porque serão várias iterações para tratar)
        })
    }
}
