/* eslint-disable no-return-await */

import requesModule from '@/assets/libs/requestModule'
import idxdDB from '@/assets/libs/IndexedDBController'
import Util from '@/assets/libs/util'
import CartController from '@/assets/controllers/CartController'
import IdentifiedConsumerController from '@/assets/controllers/IdentifiedConsumerController'
import eventModule from '@/assets/libs/eventModule'

let products = []
let productsList = {}
let showOnce = false

let getGeneralInfo = () => {
    return {...Util.getGeneralInfo()}
}

let cacheServiceUrl = Util.getCacheServiceUrl()

// let timeout = 20 * 60 * 1000
// let makeProdRequest = () => requesModule.postRequest('v3/menu/getByPosExFull', getGeneralInfo(), 'parent', (timeout)).then(r => r && r.data) // menu/getByPosEx
let makeProdRequest = () => Util.rawAxiosRequest(cacheServiceUrl, false, 'getcachedfoodmenu', 'post', getGeneralInfo())

let manageDB = async () => {
    if (Util.getLocalConfigs('urlOdhenInterface') != null) {
        if (window.indexedDB) {
            console.log('Your browser support a stable version of IndexedDB.')
            let checkTime = Util.isTimeToReloadFoodMenu()
            if (checkTime || !products.length) {
                let db = await idxdDB.getDb()
                products = await idxdDB.getProdSIsFromDb(db)
                products = products && products.length ? products[0] : []

                if (checkTime || !products.length) {
                    return Promise.all([makeProdRequest(), idxdDB.deleteProdSIFromDb(0, db)]).then(values => {
                        showOnce = false
                        // remove empty groups from menu
                        products = (JSON.parse(values[0].result).data.groups || []).filter(cat => ((cat.subGroups && cat.subGroups.length > 0) || (cat.products && cat.products.length > 0 && cat.products.length === cat.products.map(p => p.productShowOnMenu === 'N').length)))
                        // products = (JSON.parse(values[0].result).data.groups || []).filter(cat => ((cat.subGroups && cat.subGroups.length > 0) || (cat.products && cat.products.length > 0)))
                        productsList = Util.generateProdList(products)

                        return idxdDB.addProdSIToDb(products, db).then(() => {
                            console.log('External DB was requested')
                            Util.updateLastSync()
                            return products
                        })
                    }).catch(e => {
                        if (e && e.message && (showOnce !== e.message)) {
                            let msg = e.message
                            showOnce = msg
                            Util.showInfoModal(msg)
                            Util.setLocalVar('lastSync', 'XI')
                            console.log(e)
                            return manageDB()
                        }
                    })
                }
            }
            console.log('Local database loaded')

            if (Object.values(productsList).length === 0) {
                productsList = Util.generateProdList(products)
            }
            return products
        }
        return makeProdRequest()
    }
    console.error('Odhen Interface url is missing')
    Util.showInfoModal('404 - Existem configurações faltantes!')
}

let getProductsFromComanda = async function (comandaNumber, resetCart = true) {
    if (resetCart) {
        CartController.resetCart()
    }
    let requestObject = getGeneralInfo()
    requestObject.saleDescription = comandaNumber
    let retorno = await requesModule.postRequest('sale/getOrderData',
        requestObject,
        'parent').then(r => r).catch((e) => e)

    if (retorno && retorno.error) {
        let msg = retorno.message
        Util.showInfoModal(msg)
        if (Util.isTapMode()) {
            setTimeout(() => {
                eventModule.$modal.hideAll()
            }, 3000)
            eventModule.$emit('restartListenForRfid')
        }
    } else {
        let orderData = (retorno && retorno.data && retorno.data.order) || []
        let comandasAgregadas = CartController.getComandasAgregadas()
        if (comandasAgregadas.length === 0 && orderData.consumers != null && orderData.consumers[0].clientCode != null && orderData.consumers[0].consumerCode != null) {
            IdentifiedConsumerController.setConsumerDataComanda(orderData.consumers[0].clientCode, orderData.consumers[0].consumerCode, comandaNumber, orderData.saleCode)
        }
        if (orderData.consumers) {
            IdentifiedConsumerController.setConsumersList(orderData.consumers)
        }
        orderData.items.forEach(item => {
            console.log(item)
            item.totalItemPrice = item.itemValue || Util.safeDecimalCalc(item.unitValue, item.discount, '-')
            item.unitValue = item.unitValue
            item.observations = item.observations || []
            item.productCombo = item.productCombo || []
            item.comandaNumber = item.comandaNumber || orderData.saleCode
            CartController.addToCart(item)
        })
        CartController.setComanda(orderData)
    }
    return retorno.error
}

let getProdItem = function (key) {
    if (!(productsList && productsList.length)) {
        return []
    }
    if (key === 'unique') {
        return [...new Map(productsList.map((item) => [(item.productCode || item.subItemCode), item])).values()]
    }
    return key ? productsList.find(p => (p.productCode === key)) : productsList
}

let getBlockProducts = async function () {
    let requestObject = getGeneralInfo()
    return await requesModule.postRequest('v3/menu/getBloqProducts',
        requestObject,
        'parent').then(r => (r && r.data) || []).catch((e) => [])
}

let getComandaPai = function () {
    let comandas = Util.deepCopy(CartController.getComandasAgregadas())
    return comandas && comandas[0]
}

let splitComandas = async function (saleDescription) {
    let requestObject = getGeneralInfo()
    requestObject.saleDescription = saleDescription
    let child = await requesModule.postRequest('sale/getOrderData',
        requestObject,
        'parent').then(r => r).catch((e) => e)
    if (child.error) {
        let msg = child.message
        Util.showInfoModal(msg)
        return child.error
    } else {
        let childOrderData = (child && child.data && child.data.order) || []
        requestObject = null
        requestObject = getGeneralInfo()
        requestObject.saleCode = getComandaPai().saleCode
        requestObject.childSaleCode = childOrderData.saleCode

        let retorno = await requesModule.postRequest('sale/splitOrders',
            requestObject,
            'parent').then(r => r).catch((e) => e)
        if (retorno.error) {
            let msg = retorno.message
            Util.showInfoModal(msg)
        } else if (CartController.getMainComandaAgregada().serviceTax !== null) {
            let mainComandaTax = CartController.getMainComandaAgregada().serviceTax
            let subComandaTax = childOrderData.serviceTax
            CartController.getMainComandaAgregada().serviceTax = Util.safeDecimalCalc(mainComandaTax, subComandaTax, '-')
        }
        CartController.filterComanda(childOrderData.saleCode)
        return retorno.error
    }
}

let joinComandas = async function (childSaleDescription) {
    let requestObject = getGeneralInfo()
    requestObject.saleDescription = childSaleDescription
    let retorno = await requesModule.postRequest('sale/getOrderData',
        requestObject,
        'parent').then(r => r).catch((e) => e)
    if (retorno.error) {
        let msg = retorno.message
        Util.showInfoModal(msg)
        return retorno.error
    }
    let childSaleCode = retorno.data.order.saleCode

    requestObject = {}
    requestObject = getGeneralInfo()
    requestObject.saleCode = getComandaPai().saleCode
    requestObject.childSaleCode = childSaleCode

    retorno = {}
    retorno = await requesModule.postRequest('sale/joinOrders',
        requestObject,
        'parent').then(r => r).catch((e) => e)
    if (retorno.error) {
        let msg = retorno.message
        Util.showInfoModal(msg)
    }
    return retorno.error
}

let addRemoveBloqProducts = async function (code, status) {
    let route = status ? 'menu/addBloqProducts' : 'menu/removeBloqProducts'
    return await requesModule.postRequest(route,
        {
            subsidiaryCode: Util.getGeneralInfo()['subsidiaryCode'],
            storeCode: Util.getGeneralInfo()['storeCode'],
            products: [code]
        },
        'parent').then(r => r)
}

export default {
    addRemoveBloqProducts,
    getProductsFromComanda,
    splitComandas,
    joinComandas,
    getProducts: () => manageDB(),
    getBlockProducts,
    getProductObservations: function (productCode) {
        let requestObject = getGeneralInfo()
        requestObject.productCode = productCode
        return requesModule.postRequest('menu/getProductObservations',
            requestObject,
            'parent').then(r => r && r.data)
    },
    getProductCombos: function (productCode) {
        let requestObject = getGeneralInfo()
        requestObject.productCode = productCode
        return requesModule.postRequest('menu/getProductCombos',
            requestObject,
            'parent').then(r => r && r.data)
    },
    getProdItem
}
