/* eslint-disable */

/* eslint-disable no-return-await */
import Util from '@/assets/libs/util'
import requesModule from '@/assets/libs/requestModule'
import CartController from '@/assets/controllers/CartController'
import PrinterController from '@/assets/controllers/PrinterController'
import PaymentController from '@/assets/controllers/PaymentController'
import IdentifiedConsumerController from '@/assets/controllers/IdentifiedConsumerController'

// const EVENT_SAVE_ORDER = 'sale/saveDeliverySale' // rota padrao da Odhen Interface
const EVENT_SAVE_ORDER = 'v3/sale/saveDeliverySaleEx' // rota criada para o novo TAA (retorna o pagerID randômico e aplica regra de imprime sim/não)
const EVENT_CLOSE_ORDER = 'sale/closeOrder'
const EVENT_GET_SALE = 'sale/getSale'
const EVENT_CANCEL_ORDER = 'sale/cancelOrder'
const EVENT_CANCEL_PAYMENT = 'sale/cancelPayments'
const EVENT_CANCEL_SALE = 'sale/cancelSale'
const EVENT_SAVE_PAYMENT = 'sale/savePayment'
const EVENT_CLEAR_COMANDA = 'sale/cleanOrder'
const EVENT_EDIT_SERVICE_TAX = 'sale/editServiceTax'

const buildSaleData = (paymentNumber, paymentMethod, cart, orderCPF = '') => {

    console.log('Carrinho')
    console.log(cart)

    let isBalancaMode = Util.isBalancaMode()
    let orderData = {}
    let dataObject = {
        taaSaleId: '', // Hash pra identificação (opcional)
        order: orderData // dados da venda
    }

    // Dados gerais da venda
    if (paymentMethod.IDTIPORECE === '4' || Util.isTapMode()) {
        orderData.closeByApi = false
    } else {
        orderData.closeByApi = true
    }
    orderData.subsidiaryCode = Util.getGeneralInfo()['subsidiaryCode']
    orderData.storeCode = Util.getGeneralInfo()['storeCode']
    orderData.posCode = Util.getGeneralInfo()['posCode']
    orderData.pagerId = CartController.getPagerCode() !== null ? CartController.getPagerCode() : ''
    orderData.clientCode = ''
    orderData.consumerCode = ''
    orderData.consumerId = ''
    orderData.items = []

    // Detalhes da venda
    orderData.orderDescription = {}

    let modoentrega = Util.getLocalVar('modoentrega')
    let toTravel = modoentrega === 'levar'
    let driveThru = modoentrega === 'drive'

    orderData.orderDescription.toTravel = toTravel
    orderData.orderDescription.driveThru = driveThru
    //orderData.orderDescription.orderType = modoentrega === 'drive' ? 'DRV' : 'TAA' 
    orderData.orderDescription.orderType = 'TAA' 
    
    let totalPrice
    let totalPriceStr = CartController.calculateTotalPrice()
    totalPriceStr = totalPriceStr.replace('R$', '').replace(',', '').replace('.', '').trim()
    totalPrice = parseFloat(totalPriceStr) / 100

    orderData.orderDescription.productsValue = totalPrice
    orderData.orderDescription.deliveryFee = 0
    orderData.orderDescription.totalValue = orderData.orderDescription.productsValue + orderData.orderDescription.deliveryFee
    orderData.orderDescription.changeValue = 0
    orderData.orderDescription.serviceTax = 0
    orderData.orderDescription.giftCards = []
    orderData.orderDescription.deliveryAddress = null
    orderData.orderDescription.isScheduled = false
    orderData.orderDescription.scheduledDate = ''
    orderData.orderDescription.orderCPF = ''
    if (orderCPF !== '') {
        // A regex retira da string tudo que nao sejam numeros
        orderCPF = orderCPF.replace(/\D/g, '')
        orderData.orderDescription.orderCPF = orderCPF
    }

    // Trata do pagamento para esta venda
    if (paymentMethod.IDTIPORECE === '4') {
        orderData.orderDescription.payments = []
    } else {
        orderData.orderDescription.payments = [{
            paymentCode: generatePaymentCode(paymentMethod),
            paymentName: '', // Descrição do pagamento
            value: totalPrice,
            nsu: paymentNumber // Codigo do pagamento retornado pelo TEF. Em caso de pagamento em dinheiro, chega aqui como string vazia
        }]
    }

    // Monta os itens da venda, de acordo com o Carrinho
    cart.forEach(product => {
        let current = {}
        current.productName = product.productName
        current.productCode = product.productCode
        current.productPrint  = product.productPrint
        current.quantity = isBalancaMode ? product.quantity : 1
        current.unitValue = product.price
        current.discount = 0
        current.toTravel = toTravel
        current.note = product.typedObs || ''
        current.observations = []
        current.childItems = []

        // Trata as observacoes para este produto (produto pai ou item simples)
        product.observations.forEach(observation => {
            if (observation.quantity === undefined) {
                observation.quantity = 1
            }
            current.observations.push(observation)
        })

        // Trata os itens filhos deste produto (quando for combo)
        product.productCombo.forEach(childItem => {
            let newChild = {
                productCode: childItem.subItemCode,
                productName: childItem.subItemName,
                productPrint: childItem.subItemProductPrint,
                groupPrint: childItem.groupPrint,
                unitValue: childItem.price,
                toTravel: toTravel,
                quantity: 1,
                discount: 0,
                note: childItem.typedObs || '',
                observations: []
            }

            // Trata as observacoes dos itens filhos (quando for combo)
            if (childItem && childItem.observations) {
                childItem.observations.forEach(obs => {
                    let newObs = {
                        obsGroupCode: obs.subItemObsGroupCode,
                        obsCode: obs.subItemObsCode,
                        obsName: obs.subItemObsName,
                        price: obs.price || 0,
                        quantity: obs.quantity || 1,
                        obsType: obs.subItemObsType,
                        obsProductCode: obs.subItemObsProductCode
                    }
                    newChild.observations.push(newObs)
                })
            }

            // Trata os itens filhos de filhos quando for combo do combo
            if (childItem.choosen) {
                newChild.childItems = []
                let keys = Object.keys(childItem.choosen)
                keys.forEach(key => {
                    childItem.choosen[key].forEach(subSubProd => {
                        let newInnerChild = {
                            productCode: subSubProd.subItemCode,
                            productName: subSubProd.subItemName,
                            productPrint: subSubProd.groupPrint,
                            unitValue: subSubProd.price,
                            toTravel: toTravel,
                            quantity: 1,
                            discount: 0,
                            note: subSubProd.typedObs || '',
                            observations: []
                        }

                        // Trata as observacoes dos itens filhos de filhos (quando for combo do combo)
                        if (subSubProd && subSubProd.observations) {
                            subSubProd.observations.forEach(obs => {
                                let newObs = {
                                    obsGroupCode: obs.subItemObsGroupCode,
                                    obsCode: obs.subItemObsCode,
                                    obsName: obs.subItemObsName,
                                    price: obs.price || 0,
                                    quantity: obs.quantity || 1,
                                    obsType: obs.subItemObsType,
                                    obsProductCode: obs.subItemObsProductCode
                                }
                                newInnerChild.observations.push(newObs)
                            })
                        }
                        let subQuantity = subSubProd.quantity ? subSubProd.quantity : 1
                        for (let i = 0; i < subQuantity; i++) {
                            newChild.childItems.push(newInnerChild)
                        }
                    })
                })
            }
            let quantity = childItem.quantity ? childItem.quantity : 1
            for (let i = 0; i < quantity; i++) {
                current.childItems.push(newChild)
            }
            
        })
        if (isBalancaMode) {
            orderData.items.push(current)
        } else {
            let fatherQuantity = product.quantity ? product.quantity : 1
            for (let i = 0; i < fatherQuantity; i++) {
                orderData.items.push(current)
            }
        }
    })

    return dataObject
}

// Gera o código do pagamento de adordo com a bandeira do cartão usado
function generatePaymentCode (paymentMethod) {
    let paymentCode = null
    let bandeiraCartao = CartController.getbandeiraCartao()
    let tiposRecebimento = Util.getLocalConfigs('tiposRecebimento')

    let usedPaymentMethod = (tiposRecebimento || []).find(element => element.CDBANCARTCR != null &&
        element.CDBANCARTCR === bandeiraCartao &&
        element.CDTIPORECE === paymentMethod.CDTIPORECE
    )

    if (usedPaymentMethod === undefined) {
        usedPaymentMethod = (tiposRecebimento || []).find(element => element.CDBANCARTCR != null && element.CDBANCARTCR === bandeiraCartao)
    }

    if (usedPaymentMethod !== undefined) {
        paymentCode = usedPaymentMethod.CDTIPORECE
    } else {
        paymentCode = paymentMethod.CDTIPORECE
    }
    
    return paymentCode
}

function delay (seconds) {
    return new Promise(function (resolve) {
        setTimeout(resolve, seconds * 1000)
    })
}

const saveOrder = async (paymentNumber, paymentMethod, cart, orderCPF) => {
    return await requesModule.postRequest(EVENT_SAVE_ORDER,
        buildSaleData(paymentNumber, paymentMethod, cart, orderCPF),
        'parent').then(r => r)
}

const closeOrder = async (saleCode) => {
    let payload = {
        subsidiaryCode: Util.getGeneralInfo()['subsidiaryCode'],
        saleCode: saleCode,
        posCode: Util.getGeneralInfo()['posCode']
    }
    if (Util.isTapMode()) {
        payload.checkCloseByApi = false
    }
    return await requesModule.postRequest(EVENT_CLOSE_ORDER,
        payload,
        'parent').then(r => r)
}

const getSale = async (saleCode) => {
    return await requesModule.postRequest(EVENT_GET_SALE,
        {
            subsidiaryCode: Util.getGeneralInfo()['subsidiaryCode'],
            saleCode: saleCode,
            posCode: Util.getGeneralInfo()['posCode']
        },
        'parent').then(r => r)
}

const cancelOrder = async (saleCode) => {
    return await requesModule.postRequest(EVENT_CANCEL_ORDER,
        {
            subsidiaryCode: Util.getGeneralInfo()['subsidiaryCode'],
            saleCode: saleCode,
            posCode: Util.getGeneralInfo()['posCode']
        },
        'parent').then(r => r)
}

const cancelSale = async (nfceCode) => {
    return await requesModule.postRequest(EVENT_CANCEL_SALE,
        {
            subsidiaryCode: Util.getGeneralInfo()['subsidiaryCode'],
            posCode: Util.getGeneralInfo()['posCode'],
            nfceCode: nfceCode,
            saleSeq: null
        },
        'parent').then(r => r)
}

const savePaymentsTAP = async (saleCode, paymentMethod, payments) => {
    let personalCreditPayment = paymentMethod.IDTIPORECE === '9' || paymentMethod.IDTIPORECE === 'A'

    let data = {
        subsidiaryCode: Util.getGeneralInfo()['subsidiaryCode'],
        saleCode: saleCode,
        giftCard: [],
        cashBack: 0,
        payments: payments
    }

    if (personalCreditPayment) {
        let consumerData = IdentifiedConsumerController.getConsumerData()
        data.payments[0].clientCode = consumerData.clientCode
        data.payments[0].consumerCode = consumerData.consumerCode
    }

    return await requesModule.postRequest(EVENT_SAVE_PAYMENT, data, 'parent').then(r => r)
}

const cancelPaymentsTAP = async (saleCode) => {
    return await requesModule.postRequest(EVENT_CANCEL_PAYMENT,
        {
            subsidiaryCode: Util.getGeneralInfo()['subsidiaryCode'],
            saleCode: saleCode
        },
        'parent').then(r => r)
}

const clearComanda = async (saleCode) => {
    return await requesModule.postRequest(EVENT_CLEAR_COMANDA,
        {
            subsidiaryCode: Util.getGeneralInfo()['subsidiaryCode'],
            saleCode: saleCode
        },
        'parent').then(r => r)
}

const editServiceTax = async (saleCode, serviceTax) => {
    return await requesModule.postRequest(EVENT_EDIT_SERVICE_TAX,
        {
            subsidiaryCode: Util.getGeneralInfo()['subsidiaryCode'],
            saleCode: saleCode,
            serviceTax: serviceTax
        },
        'parent').then(r => r)
}

const trataNotaFiscalNaoRecebida = () => {
    Util.showInfoModal('Não foi possível receber o XML da nota fiscal através da Odhen Interface')
    if (Util.isTapMode()) {
        // Chamar o novo endpoint que será criado para a interface limpar a MOVCAIXA_DLV
        // Quando adicionar o endpoint, essa função terá que virar async e adicionar um await
        // na chamada do endpoint e em todas as chamadas dessa função
    }
}

const doPersonalCreditSale = async (paymentMethod, orderCPF, router, complementedPayment) => {
    let cart = CartController.getCart()
    let xml = '' // Recebe o XML da Receita
    let saleData = {} // Recebe os dados da venda da Odhen Interface

    let saveOrderResponse = null
    let maxTries = 3
    let tries = 1
    let success = false
    let saleCode = null
    let saveOrderErrorMessage = null
    let paymentNumber = ''
    let paymentOK= false

    if (Util.isTapMode()) {
        saveOrderResponse = CartController.getMainComandaAgregada()
        let totalPriceStr = CartController.calculateTotalPrice()
        totalPriceStr = totalPriceStr.replace('R$', '').replace(',', '').replace('.', '').trim()
        let totalPrice = parseFloat(totalPriceStr) / 100
        
        let serviceTax = saveOrderResponse.serviceTax
        if (saveOrderResponse.serviceTax !== null) {
            let saveeditServiceTax = await editServiceTax (saveOrderResponse.saleCode, serviceTax)
            console.log('Save Edit ServiceTax .' + serviceTax)
            console.log(saveeditServiceTax)
        }

        let paymentsArray = []

        if (complementedPayment) {
            let compPaymentData = IdentifiedConsumerController.getCompPaymentData()
            let principalPayment = {
                paymentCode: generatePaymentCode(paymentMethod),
                paymentName: '',
                value: Math.round((parseFloat(totalPrice) - parseFloat(compPaymentData.compValue)) * 100) / 100,
                nsu: paymentNumber
            }
            let complementaryPayment = {
                paymentCode: generatePaymentCode(compPaymentData.compPaymentMethod),
                paymentName: '',
                value: compPaymentData.compValue,
                nsu: compPaymentData.NSU
            }
            paymentsArray.push(principalPayment)
            paymentsArray.push(complementaryPayment)
        } else {
            paymentsArray = [{
                paymentCode: generatePaymentCode(paymentMethod),
                paymentName: '',
                value: totalPrice,
                nsu: paymentNumber
            }]
        }

        await savePaymentsTAP(saveOrderResponse.saleCode, paymentMethod, paymentsArray)
    } else {
        // do {
        //     console.log(`Tentando salvar o pedido na Odhen Interface. Tentativa ${tries} de ${maxTries}.`)
        //     // Salva o pedido através da Odhen Interface
        //     try {
        //     // throw 'testException'
    
        //         saveOrderResponse = await saveOrder(paymentNumber, paymentMethod, cart, orderCPF)
        //         console.log('Save Order Response')
        //         console.log(saveOrderResponse)
        //     } catch (error) {
        //         let message = 'Erro ao salvar a venda na Odhen Interface'
        //         console.log(message)
        //         console.log(error)
        //         if (error.error) {
        //             saveOrderErrorMessage = error.message
        //         }
        //         Util.logComunication('Exception ', {}, message)
        //     }
    
        //     if (saveOrderResponse !== null) {
        //         success = true
        //     } else {
        //         tries++
        //     }
        // } while (tries <= maxTries && !success)

        // if (!success && saveOrderErrorMessage) {
        //     Util.showInfoModal(saveOrderErrorMessage)
        // }
    }
    if (saveOrderResponse !== null) {
        // Se a ordem foi salva corretamente, solicita à Odhen Interface o fechamento da comanda
        if (isValidResponse(saveOrderResponse)) {
            saveOrderResponse = saveOrderResponse.data ? saveOrderResponse.data[0] : saveOrderResponse

            CartController.setPagerCode(saveOrderResponse.pagerId)

            saleCode = saveOrderResponse.saleCode
            let closeOrderResponse = await closeOrder(saleCode)

            // Se o pedido de fechamento nao retornou erro, aguarda até que o fechamento seja finalizado e o XML
            // da nota fiscal seja recebido
            let orderAlreadyClosed = closeOrderResponse.data && closeOrderResponse.data.orderAlreadyClosed === true
            if (!closeOrderResponse.error || orderAlreadyClosed) {
                let getSaleResponse
                let numberOfTries = 0
                const maxTries = 23

                do {
                    numberOfTries++
                    await delay(2)
                    getSaleResponse = await getSale(saleCode)

                    if (getSaleResponse.error) {
                        Util.showInfoModal(getSaleResponse.message)
                        break
                    }

                } while (getSaleResponse.data.length === 0 && numberOfTries < maxTries)

                // Se saiu do do-while porque recebeu a resposta esperada, continua a operação de venda
                if (getSaleResponse.data.length !== 0) {
                    xml = getSaleResponse.data.xml
                    saleData = getSaleResponse.data

                    // Se um XML de nota fiscal foi recebido, imprime os comprovantes para o cliente
                    if (xml !== '') {
                        console.log('Nota fiscal recebida com sucesso')
                        paymentOK = true

                        if (complementedPayment) {
                            let compPaymentData = IdentifiedConsumerController.getCompPaymentData()
                            // 1 = confirma transacao
                            await PaymentController.finishTransactionTEF(1, '', compPaymentData.date, compPaymentData.hour)
                            console.log('CP/DC - Transacao finalizada com sucesso')
                            CartController.clearbandeiraCartao()
                        }

                        if (Util.isTapMode()) {
                            await clearComanda(saveOrderResponse.saleCode)
                        }

                        return { 
                            print: true,
                            printData: {
                                saleData: saleData,
                                xml: xml,
                                paymentMethod: paymentMethod,
                                paymentNumber: paymentNumber
                            }
                        }
                    }
                }

                if (xml === '') {
                    trataNotaFiscalNaoRecebida()
                }
            }
            if (closeOrderResponse.error && !orderAlreadyClosed) {
                Util.showInfoModal(closeOrderResponse.message)
            }
        } else {
            Util.showInfoModal('Não foi possível completar a venda. O valor pago foi estornado.')
        }
    }

    if (!paymentOK) {
        if (saleCode !== null) {
            if (!Util.isTapMode()) {
                // Cancela a comanda/venda através da Odhen Interface se nao for TAP
                // console.log(`Solicitando o cancelamento da comanda numero ${saleCode}`)
                // let cancelResponse = await cancelOrder(saleCode)
                // console.log(cancelResponse)
            } else {
                // Cancela a pagamentos através da Odhen Interface se for TAP
                console.log('Solicitando o cancelamento do pagamento ' + saleCode)
                let cancelPaymentResponse = await cancelPaymentsTAP(saleCode)
                console.log(cancelPaymentResponse)
            }
        }

        if (complementedPayment) {
            let compPaymentData = IdentifiedConsumerController.getCompPaymentData()
            // 0 = não confirma transacao
            await PaymentController.finishTransactionTEF(0, '', compPaymentData.date, compPaymentData.hour)
            let message = 'CP/DC - Transacao interrompida na rotina de salvarmento da venda com estorno do valor'
            console.log(message)
            Util.logComunication('Exception ', {}, message)
            CartController.clearbandeiraCartao()
        }
    }

    // Embora este bloco verifique uma codição oposta à condição usada por um bloco if anterior (aqui se verifica
    // saveOrderResponse === null e lá se verifica saveOrderResponse !== null) esta chamada de showErrorScreen
    // NÃO PODE se transformar em um else do primeiro if, porque entre uma verificação e outra é preciso ter o bloco
    // acima que faz o cancelamento da transação.
    if (saveOrderResponse === null) {
        return Util.showErrorScreen(router, 'saveSaleError')
    }

    CartController.clearCartAndNavigateHome(router)
}

const doCashSale = async (paymentMethod, orderCPF, router) => {
    let cart = CartController.getCart()
    let saveOrderResponse = null
    let maxTries = 3
    let tries = 1
    let success = false
    let saveOrderErrorMessage = null

    do {
        console.log(`Tentando salvar o pedido na Odhen Interface. Tentativa ${tries} de ${maxTries}.`)
        // Salva o pedido, ainda pendente de pagamento, através da Odhen Interface.
        try {
            // throw 'testException'
            // O primeiro parametro seria o numero do pagamento, por isso é passada uma string vazia.
            saveOrderResponse = await saveOrder('', paymentMethod, cart, orderCPF)
        } catch (error) {
            let message = 'Erro ao salvar a venda na Odhen Interface'
            console.log(message)
            console.log(error)
            if (error.error) {
                saveOrderErrorMessage = error.message
            }
            Util.logComunication('Exception ', {}, message)
        }

        if (saveOrderResponse !== null) {
            success = true
        } else {
            tries++
        }
    } while (tries <= maxTries && !success)

    if (!success && saveOrderErrorMessage) {
        Util.showInfoModal(saveOrderErrorMessage)
    }

    if (saveOrderResponse !== null) {
        if (isValidResponse(saveOrderResponse)) {

            let pagerId = saveOrderResponse.data[0].pagerId
            CartController.setPagerCode(pagerId)
            let saleData = {
                pagerId: pagerId
            }

            let printCouponResult = await PrinterController.printCoupon(saleData)

            if (!printCouponResult || (printCouponResult && printCouponResult.data.error)) {
                console.log('Falha na impressora. Não foi possível imprimir o cupom do cliente.')
                return Util.showErrorScreen(router, 'printerError')
            }
        } else {
            Util.showInfoModal('Não foi possível completar a venda.')
        }
    } else {
        return Util.showErrorScreen(router, 'saveSaleError')
    }

    CartController.clearCartAndNavigateHome(router)
}

const doTefSale = async (paymentNumber, paymentMethod, orderCPF, router, paymentDate, paymentHour) => {
    let cart = CartController.getCart()
    let xml = '' // Recebe o XML da Receita
    let saleData = {} // Recebe os dados da venda da Odhen Interface

    let saveOrderResponse = null
    let maxTries = 3
    let tries = 1
    let success = false
    let confirmTEF = false
    let saleCode = null

    let saveOrderErrorMessage = null

    if (Util.isTapMode()) {
        saveOrderResponse = CartController.getMainComandaAgregada()
        let totalPriceStr = CartController.calculateTotalPrice()
        totalPriceStr = totalPriceStr.replace('R$', '').replace(',', '').replace('.', '').trim()
        let totalPrice = parseFloat(totalPriceStr) / 100
        
        let serviceTax = saveOrderResponse.serviceTax
        if (saveOrderResponse.serviceTax !== null) {
            await editServiceTax (saveOrderResponse.saleCode, serviceTax)
        }

        await savePaymentsTAP(saveOrderResponse.saleCode, paymentMethod, [{
            paymentCode: generatePaymentCode(paymentMethod),
            paymentName: '', // Descrição do pagamento
            value: totalPrice,
            nsu: paymentNumber // Codigo do pagamento retornado pelo TEF. Em caso de pagamento em dinheiro, chega aqui como string vazia
        }])
    } else {
        do {
            console.log(`Tentando salvar o pedido na Odhen Interface. Tentativa ${tries} de ${maxTries}.`)
            // Salva o pedido através da Odhen Interface
            try {
                // throw 'testException'
                saveOrderResponse = await saveOrder(paymentNumber, paymentMethod, cart, orderCPF)
            } catch (error) {
                let message = 'Erro ao salvar a venda na Odhen Interface'
                console.log(message)
                console.log(error)
                if (error.error) {
                    saveOrderErrorMessage = error.message
                }
                Util.logComunication('Exception ', {}, message)
            }
    
            if (saveOrderResponse !== null) {
                success = true
            } else {
                tries++
            }
        } while (tries <= maxTries && !success)

        if (!success && saveOrderErrorMessage) {
            Util.showInfoModal(saveOrderErrorMessage)
        }
    }
    if (saveOrderResponse !== null) {
        // Se a ordem foi salva corretamente, solicita à Odhen Interface o fechamento da comanda
        if (isValidResponse(saveOrderResponse)) {
            saveOrderResponse = saveOrderResponse.data ? saveOrderResponse.data[0] : saveOrderResponse

            CartController.setPagerCode(saveOrderResponse.pagerId)

            saleCode = saveOrderResponse.saleCode
            let closeOrderResponse = await closeOrder(saleCode)

            // Se o pedido de fechamento nao retornou erro, aguarda até que o fechamento seja finalizado e o XML
            // da nota fiscal seja recebido
            let orderAlreadyClosed = closeOrderResponse.data && closeOrderResponse.data.orderAlreadyClosed === true
            if (!closeOrderResponse.error || orderAlreadyClosed) {
                let getSaleResponse
                let numberOfTries = 0
                const maxTries = 23

                do {
                    numberOfTries++
                    await delay(2)
                    getSaleResponse = await getSale(saleCode)

                    if (getSaleResponse.error) {
                        Util.showInfoModal(getSaleResponse.message)
                        break
                    }

                } while (getSaleResponse.data.length === 0 && numberOfTries < maxTries)

                // Se saiu do do-while porque recebeu a resposta esperada, continua a operação de venda
                if (getSaleResponse.data.length !== 0) {
                    xml = getSaleResponse.data.xml
                    saleData = getSaleResponse.data

                    // Se um XML de nota fiscal foi recebido, finaliza a rotina do TEF com confirmação e imprime os
                    // comprovantes para o cliente
                    if (xml !== '') {
                        console.log('Nota fiscal recebida com sucesso')
                        // 1 = confirma transacao
                        await PaymentController.finishTransactionTEF(1, '', paymentDate, paymentHour)
                        confirmTEF = true
                        console.log('Transacao finalizada com sucesso')
                        CartController.clearbandeiraCartao()

                        if (Util.isTapMode()) {
                            await clearComanda(saveOrderResponse.saleCode)
                        }

                        return { 
                            print: true,
                            printData: {
                                saleData: saleData,
                                xml: xml,
                                paymentMethod: paymentMethod,
                                paymentNumber: paymentNumber
                            }
                        }
                    }
                }

                if (xml === '') {
                    trataNotaFiscalNaoRecebida()
                }
            }
            if (closeOrderResponse.error && !orderAlreadyClosed) {
                Util.showInfoModal(closeOrderResponse.message)
            }
        } else {
            Util.showInfoModal('Não foi possível completar a venda. O valor pago foi estornado.')
        }
    }

    if (!confirmTEF) {
        // 0 = nao confirma transacao e estorna o valor
        await PaymentController.finishTransactionTEF(0, '', paymentDate, paymentHour)
        let message = 'Transacao interrompida na rotina de salvarmento da venda com estorno do valor'
        console.log(message)
        Util.logComunication('Exception ', {}, message)
        CartController.clearbandeiraCartao()

        if (saleCode !== null) {
            if (!Util.isTapMode()) {
                // Cancela a comanda/venda através da Odhen Interface se nao for TAP
                console.log(`Solicitando o cancelamento da comanda numero ${saleCode}`)
                let cancelResponse = await cancelOrder(saleCode)
                console.log(cancelResponse)
            } else {
                // Cancela a pagamentos através da Odhen Interface se for TAP
                console.log('Solicitando o cancelamento do pagamento ' + saleCode)
                let cancelPaymentResponse = await cancelPaymentsTAP(saleCode)
                console.log(cancelPaymentResponse)
            }
        }
    }

    // Embora este bloco verifique uma codição oposta à condição usada por um bloco if anterior (aqui se verifica
    // saveOrderResponse === null e lá se verifica saveOrderResponse !== null) esta chamada de showErrorScreen
    // NÃO PODE se transformar em um else do primeiro if, porque entre uma verificação e outra é preciso ter o bloco
    // acima que faz o cancelamento da transação TEF.
    if (saveOrderResponse === null) {
        return Util.showErrorScreen(router, 'saveSaleError')
    }

    CartController.clearCartAndNavigateHome(router)
}

const isValidResponse = function (saveOrderResponse) {
    return (!saveOrderResponse.error && saveOrderResponse.data && saveOrderResponse.data[0] && saveOrderResponse.data[0].saleCode && saveOrderResponse.data[0].saleCode.length > 0)
        || (saveOrderResponse.saleCode && saveOrderResponse.saleCode.length > 0)
}

export default {
    performSale: async function (paymentNumber, paymentMethod, orderCPF, router, paymentDate, paymentHour, complementedPayment = false) {
        if (paymentMethod.IDTIPORECE === '9' || paymentMethod.IDTIPORECE === 'A') {
            return await doPersonalCreditSale(paymentMethod, orderCPF, router, complementedPayment)
        } else if (paymentMethod.IDTIPORECE === '4') {
            // IDTIPORECE === '4' significa que a forma de pagamento escolhida foi dinheiro
            await doCashSale(paymentMethod, orderCPF, router)
        } else {
            return await doTefSale(paymentNumber, paymentMethod, orderCPF, router, paymentDate, paymentHour)
        }
    },
    cancelSale,
    editServiceTax
}
