/* eslint-disable*/

import axios from 'axios'
import Util from '@/assets/libs/util'

/**
 * Build request object according to request
 * type.
 * @param {Object} dataObject
 * @param {String} requestType
 * @returns {Object}
 */
const buildRequestObject = function (dataObject, requestType) {
    let requestObject = {
        requestType
    }
    
    if(requestType == 'parent'){
        requestObject = dataObject;
    }
    else{
        requestObject[requestType.toLowerCase()] = dataObject
    }

    return requestObject
}

const _getBaseUriPrefix = function () {
    return Util.getLocalConfigs('urlOdhenInterface')
}

const _getPerifericosURI = function () {
    return Util.getLocalConfigs('urlPerifericos')
}

export default {
    baseUriPrefix: _getBaseUriPrefix(),
    baseUri: process.env.VUE_APP_BASE_URI,
    perifericosURI: _getPerifericosURI(),

    postRequest: async function (url, dataObject = {}, requestType = "Row", timeout = 1200000, extraOptions = {}) {

        const requestOptions = {
            responseType: 'json',
            ...extraOptions
        }
        let urlConcat = this.baseUriPrefix + '/' + this.baseUri + '/' + url
        let interfaceRespose =  await axios({
            method: 'post',
            url: urlConcat,
            data: buildRequestObject(dataObject, requestType),
            timeout,
            ...requestOptions
        })
            .then(successResponse => {
                return successResponse && successResponse.data || successResponse;
            })
            .catch(axiosResponse => {
                throw axiosResponse;
            });
        
        
        Util.logComunication(' interface|' + url, dataObject, interfaceRespose)

        return interfaceRespose
    },
    curlRequest: function (url, dataObject = {}, testURLPerifericos) {
        // testURLPerifericos só chegará aqui sendo !== null quando se tratar de uma impressão de teste, disparada da
        // tela de Configs. Nesse caso pode ainda não haver uma URL de Perifericos configurada em this.perifericosURI e
        // será utilizada a URL que está no campo de input da tela de Configs
        let perifericosURI = testURLPerifericos ? testURLPerifericos : this.perifericosURI
        return axios.post(perifericosURI + url, 
            {
                ...dataObject
            }
        )
        .then(response => {
            return response;
        })
        .catch(function (error) {
            console.log(error)
        });
    }
}