/* eslint-disable*/
import axios from 'axios'
import moment from 'moment'
import Vue from 'vue'

let backAdress = process.env.BACKADRESS

/**
 * Debounce function, taken from Underscore.js
 * 
 * @param {Function} func 
 * @param {Number} wait 
 * @param {Boolean} immediate 
 */
var debounce = function(func, wait, immediate, context) {
    this.timeout = null;

    return function(){
        context = context || this
        var args = arguments;
        var later = function(resolve) {
            if (!immediate) return resolve(func.apply(context, args));
        };
        var callNow = immediate && !this.timeout;
        clearTimeout(this.timeout);
        
        if (callNow) {
            return Promise.resolve(func.apply(context, args));
        } else {
            return new Promise((resolve)=>{
                this.timeout = setTimeout(()=>later(resolve), wait);
            })
        }
    };
};
debounce.timeout = null;
debounce.clearDebounce = ()=>{
    clearTimeout(debounce.timeout);
};

// Get the element with the highest occurrence in an array
const mode = function (arr){
    return arr.sort((a,b) =>
          arr.filter(v => v===a).length
        - arr.filter(v => v===b).length
    ).pop();
}

const modeLength = function (arr){
    return arr.filter(e => e === mode(arr)).length
}

/**
 * @param {String} baseStr 
 * @param {String} maskStr 
 * @param {String} char 
 */
const insertChar = (baseStr, maskStr) => {
    if(baseStr === undefined || baseStr === null){
        return undefined;
    }
    var newStr = "";
    var offset = 0;

    for (var i = 0; i < maskStr.length && i < baseStr.length + offset; i++) {
        if (maskStr[i] === ".") {
            newStr += ".";
            offset++;
        } else {
            newStr += baseStr[i - offset];
        }
    }
    return newStr;
};

const maskFactory = function (maskStr) {
    return function (o) {
        const value = o.value;
        return insertChar(value, maskStr);
    }
};

const showConfirmationDialog = function(modal, wrappedCallback = () => {}, text = "?", otherOptionCallback = () => {}){
    let translated = this.translate(text)
    modal.show({
        template:`
            <div class="confirmation-dialog-box"> 
                <h5 class="title"> ${translated} </h5>
                <div class="dialog-button-container">
                <button class="button-default modal btn-outlined" @click="handleNo"> Não </button>
                <button class="button-default modal" @click="deleteValidity"> Sim </button>
                </div>    
            </div>
        `,
        methods:{
            deleteValidity(){
                wrappedCallback()
                this.close()
            },
            handleNo(){
                otherOptionCallback()
                this.close()
            },
            close(){
                this.$emit('close')
            }
        }
    }, {}, {
        height:'auto',
        width:'600px',
    });
};

const showInfoModal = function(text = ''){
    let translated = this.translate(text)
    let modal = new Vue().$modal
    let img = require('../../../static/icons/report_problem.svg')
    modal.show({
        template:`
            <div class="confirmation-dialog-box">
                <div style="display: flex; justify-content: center">
                    <img src="${img}" style="width: 96px">
                </div>
                <h5 class="title" style="padding-top: 35px"> ${translated.substring(0, 300)} </h5>
                <div class="dialog-button-container" style="display: block">
                    <button class="button-default modal" @click="close"> OK </button>
                </div>    
            </div>
        `,
        methods:{
            close(){this.$emit('close');}
        }
    },
    {},
    {
        height:'auto',
        width:'600px',
    });
};

let isInvalidNumber = (value)=> {
    return value == null || value == undefined  || value == NaN;
};

// Verifica um caso de exceção que existe na regra de imprime sim/não do FOS, que faz o preço do combo ser
// desconsiderado quando o normal, pela regra, seria considerar
const isFosException = function (product, fatherProductCode, firstLevelComboPrint, groupPrint) {
    let comboDoCombo = isComboDoCombo(product, fatherProductCode)
    let selledInsideHimself = isSelledInsideHimself(product)

    return firstLevelComboPrint && groupPrint && comboDoCombo && !selledInsideHimself
}

const isComboDoCombo = (product, fatherProductCode) => {
    return product.subItemProductGroups.length > 0 && product.subItemCode !== fatherProductCode
}

const isSelledInsideHimself = function (product) {
    let selledInsideHimself = false
    let hasProductGroups = product && product.subItemProductGroups && product.subItemProductGroups.length > 0
    if (hasProductGroups) {
        product.subItemProductGroups.forEach(subGroup => {
            let hasItems = subGroup.subProductGroupItems && subGroup.subProductGroupItems.length > 0
            if (hasItems) {
                subGroup.subProductGroupItems.forEach(subProd => {
                    if (subProd.subItemCode === product.subItemCode) {
                        selledInsideHimself = true
                    }
                })
            }
        })
    }
    return selledInsideHimself
}

/**
 * @param {Array<Object>} array 
 * @param {Array<String>} keys 
 */
const sortBy = function(array = [], keys = []) {
    return array.sort((a, b) => {
        for (var i = 0; i < keys.length; i++) {
            const k = keys[i];
            let diff = a[k] - b[k];
            if(isNaN(diff)){
                const subjectA = !!a[k] && a[k] || '';
                const subjectB = !!b[k] && b[k] || '';
                diff = subjectA.localeCompare(subjectB);
            }
            if(diff !== 0){
                return diff;
            }
        }
        return 0;
    });
};

const formatMoney = (value) => {
    return new Intl.NumberFormat('pt-BR', {style: 'currency', currency: 'BRL'}).format(value)
}

const removeBlockedProds = function (prods) {
    let blockProds = getLocalVar('blockProds') || []
    let filtredProds = (prods || []).filter(p => (blockProds.indexOf(p.subItemCode) === -1))
    return removeHiddenProducts(filtredProds)
}

const removeHiddenProducts = function (prods) {
    return (prods || []).filter(p => (p.productShowOnMenu != 'N'))  
}

/**
 * Return values stored into localStorage.
 * 
 * @param {String} varName 
 */
const getLocalVar = function (varName){
    return JSON.parse(localStorage.getItem(varName));
}

export default {
    getCacheServiceUrl() {
        if (process.env.NODE_ENV !== 'production') {
            return 'http://localhost:1812'
        }

        let apacheLocation = this.getLocalConfigs('urlIMGOdhenInterface') || ''
        let cacheServiceUrl = apacheLocation.split(':')
        if (cacheServiceUrl.length > 1) {
            cacheServiceUrl = `${cacheServiceUrl[0]}:${cacheServiceUrl[1]}:1812/`
            return cacheServiceUrl
        }
    },
    exibApartir() {
        return this.getLocalConfigs('exibApartir') == true
    },
    variablePrice: function (prod) {
        // verifica se o produto principal não imprime no cupom
        if (prod.productPrint === 'N') {
            return true
        }

        // verifica se alguma observaçao do produto principal possui preço
        if (prod.observationsChoice) {
            let observations = false
            prod.observationsChoice.forEach((e) => {
                observations = e.observations.find((obs) => {
                    return (obs.price > 0)
                })
            })
            if (observations) {
                return true
            }
        }

        // verifica os produtos filhos, se houver
        if (prod.productGroups && prod.productGroups.length > 0) {
            // verifica se algum grupo promocional imprime no cupom
            let subGroupPrint = prod.productGroups.find((e) => {
                return e.productGroupPrint === 'S'
            })
            if (subGroupPrint) return true

            // verifica se algum grupo promocional possui observações com preço
            let hasPricedObservations = false
            prod.productGroups.forEach(group => {
                if (group.productGroupItems && group.productGroupItems.length > 0) {
                    group.productGroupItems.forEach(item => {
                        if (item.subItemObservationsChoice && item.subItemObservationsChoice.length > 0) {
                            item.subItemObservationsChoice.forEach(obsGroup => {
                                if (obsGroup.subItemObservations && obsGroup.subItemObservations.length > 0) {
                                    obsGroup.subItemObservations.forEach(obs => {
                                        if (obs.price > 0) {
                                            hasPricedObservations = true
                                        }
                                    })
                                }
                            })
                        }
                    })
                }
            })
            if (hasPricedObservations) return true
        }

        return false
    },
    mountPrice: function (prod) {
        let isComboProduct = prod.productGroups && prod.productGroups.length > 0
        let priceMounted = 0
        let minPrecSubItem
        let minPrecComboCombo = {}
        let minPrecMutexGroups = {}

        // Verifica se é um produto simples ou o produto principal imprime no cupom
        if (!isComboProduct || prod.productPrint === 'S') {
            priceMounted += prod.price
        }

        // verifica se o produto principal tem subItens
        if (Object.keys(prod.productGroups || []).length > 0) {
            (prod.productGroups || []).forEach(function (group) {
                // verifica se o grupo promocional imprime no cupom e se é obrigatorio, ou se o produto principal não imprime no cupom
                if ((group.productGroupPrint === 'S' && group.productGroupMin > 0) || prod.productPrint === 'N') {
                    // pega o item de menor valor do grupo promocional
                    minPrecSubItem = removeBlockedProds(group.productGroupItems).reduce((a, b) => {
                        if (b.price < a.price) a = b
                        return a
                    })
                }
                if (minPrecSubItem) {
                    let isMutexGroup = group.productGroupMutex != null
                    if (isMutexGroup) {
                        if (minPrecMutexGroups[group.productGroupMutex]) {
                            minPrecMutexGroups[group.productGroupMutex].push(minPrecSubItem.price * group.productGroupMin)
                        } else {
                            minPrecMutexGroups[group.productGroupMutex] = []
                            minPrecMutexGroups[group.productGroupMutex].push(minPrecSubItem.price * group.productGroupMin)
                        }
                    } else {
                        // pega o valor do menor produto do grupo promocional e multiplica pela quantidade obrigatoria
                        // caso o combo não permita selecionar mais vezes o mesmo item pode apresentar divergencia no valor
                        priceMounted += (minPrecSubItem.price * group.productGroupMin)
                    }
                }

                // Trata os preços do Combo do Combo, se for o caso
                // A segunda verificação (group.productGroupName !== prod.productName) é uma solução temporária para os produtos chegando errado no
                // cardápio, trazendo o produto inteiro dentro do primeiro subgrupo. Remover assim que o problema for corrigido
                if (group.productGroupItems.length > 0 && group.productGroupName !== prod.productName) {
                    removeBlockedProds(group.productGroupItems).forEach(subGroup => {
                        let comboDoCombo = isComboDoCombo(subGroup, prod.productCode)
                        if (comboDoCombo) {
                            let paiComboDoComboImprime = prod.productPrint === 'S'
                            let minPrecSubSubItem

                            subGroup.subItemProductGroups.forEach(subSubGroup => {
                                if (group.subProductGroupMin > 0 && (subSubGroup.subProductGroupPrint === 'S' || !paiComboDoComboImprime)) {
                                    // pega o item de menor valor do grupo promocional
                                    minPrecSubSubItem = removeBlockedProds(subSubGroup.subProductGroupItems).reduce((a, b) => {
                                        if (b.price < a.price) a = b
                                        return a
                                    })
                                }
                                if (minPrecSubSubItem) {
                                    if (minPrecComboCombo[subGroup.subItemCode]) {
                                        minPrecComboCombo[subGroup.subItemCode].push(minPrecSubSubItem)
                                    } else {
                                        minPrecComboCombo[subGroup.subItemCode] = []
                                        minPrecComboCombo[subGroup.subItemCode].push(minPrecSubSubItem)
                                    }
                                }
                            })
                        }
                    })
                }

                let comboDoComboMinPrice = null
                let keys = Object.keys(minPrecComboCombo)
                keys.forEach(key => {
                    let curMinPrice = 0
                    minPrecComboCombo[key].forEach(prod => {
                        curMinPrice += prod.price
                    })
                    if (!comboDoComboMinPrice) {
                        comboDoComboMinPrice = curMinPrice
                    } else if (curMinPrice < comboDoComboMinPrice) {
                        comboDoComboMinPrice = curMinPrice
                    }
                })
                if (comboDoComboMinPrice) {
                    priceMounted += comboDoComboMinPrice * group.productGroupMin
                }
                minPrecComboCombo = {}
            })
        }

        // Trata os preços minimos de grupos que são mutuamente exclusivos
        let mutexKeys = Object.keys(minPrecMutexGroups)
        if (mutexKeys.length > 0) {
            mutexKeys.forEach(key => {
                let currentMutexGroup = minPrecMutexGroups[key]
                let currentMutexMinPrice

                currentMutexGroup.forEach((price, index) => {
                    if (index === 0) {
                        currentMutexMinPrice = price
                    } else if (price < currentMutexMinPrice) {
                        currentMutexMinPrice = price
                    }
                })
                priceMounted += currentMutexMinPrice
            })
        }

        return formatMoney(priceMounted)
    },
    rawAxiosRequest(interfaceURL, addBaseUri, uri = '', method = 'post', data = {}) {
        return axios({
            method: method,
            url: `${interfaceURL}/${addBaseUri? process.env.VUE_APP_BASE_URI : ''}${uri}`,
            data: data
        })
            .then(successResponse => {
                return successResponse && successResponse.data || successResponse;
            })
            .catch(axiosResponse => {
                throw axiosResponse;
            });
    },
    logComunication (uri, dataObject, _respose) {
        let date = moment().format('DD/MM/YYYY - HH:mm:ss')
        let payload = JSON.stringify(dataObject)
        let response = JSON.stringify(_respose)
        let log = `${date} | URI: ${uri} \nPayload: ${payload} \nResponse: ${response}\n\n\n`
        let posCode = this.getLocalConfigs('caixaCodigo')
        axios.post(backAdress + 'functions/logger.php', `posCode=${posCode}&Log=${log}`)
    },
    getAvailableInterfaces (endApache) {
        return axios.post(endApache + 'rest-api/')
    },
    getServerStoredConfig () {     
        return axios.post(backAdress + 'functions/config.php')
    },
    doLogin (data) {     
        return axios.post(backAdress + 'functions/loginAccess.php', "Pas=" + data)
    },
    getDateAsFormat(format) {
        return moment().format(format)
    },

    /**
     * Quebra um nível do objeto e o retorna como array
     * 
     * */
    flatAnObjectList: function (obj) {
        return Object.values(obj).flat(1)
     },

    /**
     * Store values into localStorage.
     * 
     * @param {String} varName 
     * @param {*} value 
     */
    setLocalVar(varName, value){
        localStorage.setItem(varName, JSON.stringify(value));
    },

    getLocalConfigs(value){
        let config = JSON.parse(localStorage.getItem('taaconfig'));
        if (config) {
            return config[value]
        }
        return null
    },

    isTapMode() {return this.getLocalConfigs('modotap') === true},

    isBalancaMode() {return this.getLocalConfigs('modobalanca') === true},

    isSlimMode() {return this.getLocalConfigs('slimMode') === true},

    isEditMode() {return this.getLocalConfigs('isEditMode') === true},

    showConsumerLogin () { return true },

    getGeneralInfo(){
        return {
            'subsidiaryCode': this.getLocalConfigs('filialCodigo'),
            'storeCode': this.getLocalConfigs('lojaCodigo'),
            'posCode': this.getLocalConfigs('caixaCodigo'),
            'consumerType': this.getLocalConfigs('consumidorTipo'),
            'hour': moment().format('HH:mm')
        }
    },

    setLocalStoredColors() {
        let config = this.getLocalVar('taaconfig')
        if(config) {
            let r =document.querySelector(':root')
            r.style.setProperty('--primary-color', config.primary_color);
            r.style.setProperty('--primary-light', config.primary_light);
            r.style.setProperty('--primary-dark', config.primary_dark);
            r.style.setProperty('--header-color', config.header_color);
            r.style.setProperty('--categorie-color', config.categorie_color);
            r.style.setProperty('--categorie-font-color', config.categorie_font_color);
            r.style.setProperty('--cart-color', config.cart_color);
        }
    },

    /**
     * Show information notification
     * 
     * @param {Object} message 
     */
    showNotification(message){
        Vue.notify({
            group: 'main',
            title: message.title,
            text: message.text
        });
    },

    /**
     * Show warning notification
     * 
     * @param {Object} message 
     */
    showWarningNotification(message){
        Vue.notify({
            group: 'main',
            type:'warn',
            title: message.title,
            text: message.text
        });
    },

    /**
     * Show error notification
     * 
     * @param {Object} message 
     */
    showErrorNotification(message){
        Vue.notify({
            group: 'main',
            type:'error',
            title: message.title || message,
            text: message.text
        });
    },

    showErrorNotificationFixed(message){
        Vue.notify({
            group: 'main',
            type: 'error',
            title: message.title,
            text: message.text,
            duration: -1
        });
    },

    /**
     * Show success notification
     * 
     * @param {Object} message 
     */
    showSuccessNotification(message){
        Vue.notify({
            group: 'main',
            type:'success',
            title: message.title,
            text: message.text
        });
    },
    
    spliceOnCondition(array, callback) {
        var i = array.length;
        while (i--) {
            if (callback(array[i], i)) {
                array.splice(i, 1);
            }
        }
    },

    arrayToOjbect(source){
        return source.reduce((acc, cur) => {
            let key = Object.keys(cur)[0];
            acc[key] = cur[key];
            return acc; 
        }, {})
    },

    safeDecimalCalc: function (param1, param2, symbol) {
        let knowFunctions = {
            '+': ((param1 * 100) + (param2 * 100))/100,
            '-': ((param1 * 100) - (param2 * 100))/100
        }
        return knowFunctions[symbol]
    },

    deepCopy(source){
        return JSON.parse(JSON.stringify(source));
    },
    isTimeToReloadFoodMenu() {
        if ((process.env.RESPECT_EXPIRATION_CONFIG === true && this.getLocalConfigs('cardapioExpira') == true) || process.env.RESPECT_EXPIRATION_CONFIG === false) {
            return moment().format('MMDDHH') != this.getLocalVar('lastSync')
        }
        return moment().format('MMDD') != this.getLocalVar('lastSync')
    },
    updateLastSync() {
        this.setLocalVar('lastSync', moment().format('MMDD'))
        if ((process.env.RESPECT_EXPIRATION_CONFIG === true && this.getLocalConfigs('cardapioExpira') == true) || process.env.RESPECT_EXPIRATION_CONFIG === false) {
            this.setLocalVar('lastSync', moment().format('MMDDHH'))
        }
    },
    mode,
    modeLength,
    sortBy,
    debounce,
    insertChar,
    maskFactory,
    showConfirmationDialog,
    showInfoModal,
    isInvalidNumber,
    formatMoney,
    isFosException,
    isComboDoCombo,
    getLocalVar,
    removeBlockedProds,
    removeHiddenProducts,
    toBase64 (file) {
        return new Promise((resolve, reject) => {
            const reader = new FileReader();
            reader.readAsDataURL(file);
            reader.onload = () => resolve(reader.result);
            reader.onerror = error => reject(error);
        })
    },
    getConfigImage (file) {
        return this.getLocalConfigs('images') && this.getLocalConfigs('images')[file] || null
    },
    getBackAdress () {
        return backAdress
    },
    generateProdList (products) {
        let startTime = moment()
        let prodList = products.map(p=> {
            if(p.products && p.products.length > 0) {
                let prods = p.products
                let subGroupProductItens = []
                prods.forEach(p=> {
                    (p.productGroups || []).forEach(pg => {
                        let pgitens = pg.productGroupItems.map(pgi => pgi).flat(1)
                        subGroupProductItens = subGroupProductItens.concat(pgitens)
                    })
                })
                return prods.concat(subGroupProductItens)
            } else if(p.subGroups &&  p.subGroups.length > 0) {
                let subGroupProduct = p.subGroups.map(s=> s.products).flat(1)
                let subGroupProductItens = []
                p.subGroups.forEach(p=> {
                    (p.productGroups || []).forEach(pg => {
                        let pgitens = pg.productGroupItems.map(pgi => pgi).flat(1)
                        subGroupProductItens = subGroupProductItens.concat(pgitens)
                    })
                })
                return subGroupProduct.concat(subGroupProductItens)
            }else {
                return []
            }
        } ).flat(1)

        // window.lista = unicos
        // .reduce((acc, curr, index) => {
        //     acc[curr.productCode] = curr;
        //     return acc;
        // }, {})
        //.forEach(function(p) {prodList[p.productCode] = p})
        var duration = moment.duration(moment().diff(startTime));
        console.log(`Indexar todos os produtos levou ${duration.asSeconds()}s`)
        return prodList
    },
    translate(text) {
        let curLang = this.getLocalVar('lang')
        let langFile = {}
        try {
            langFile = require(`../lang/${curLang}.json`)
        } catch (error) {
            langFile = {}
        }

        // only for making easy the process of listing the items to translate 
        // remove after done
        if (Object.values(langFile).length > 0 && !langFile[text]) {
            window.temquetraduzir = window.temquetraduzir || []
            if (window.temquetraduzir.indexOf(text) ===  -1) {
                window.temquetraduzir.push(text)
            }
        }
        return langFile[text] || text
    },
    removePixSaquePixTroco: function (optionsArray) {
        let filteredOptions = optionsArray.filter(option => option.name !== 'Pix Saque' && option.name !== 'Pix Troco')
        return filteredOptions
    },
    showErrorScreen: function (router, errorType) {
        router.push({
            name: 'ErrorScreen',
            params: { errorType }
        })
        return true
    }
}
