/* eslint-disable */

import Util from '@/assets/libs/util'
import requesModule from '@/assets/libs/requestModule'
import continuaTransacaoTefOBJ from '@/assets/tef/ContinuaTransacaoTefOBJ'
import eventModule from '@/assets/libs/eventModule'
import PaymentController from '@/assets/controllers/PaymentController'

const EVENT_CONTINUA_TRANSACAO = '/continue-transaction'
const STOP_TEF_COMMAND = -1

let cancelledByUser = false

const continuaTransacaoTef = async function (valorTotal, IDTIPORECE, CDTIPORECE, DATECANCEL, DOCNUMBER, CDFILIAL, CDCAIXA, isCancelTefOperation = false, saleType, isDigitalWalletPayment = false) {
    cancelledByUser = false
    try {
        let errorMessage = ''
        let comando = 0
        let tipoCampo = 0
        let tamMinimo = 0
        let tamMaximo = 0
        let bufferParam = ''
        let tamBuffer = 0
        let continua = 0

        let retornoContinua = null
        let iterationCounter = 0

        do {
            let paramsContinua = {
                command: comando,
                fieldType: tipoCampo,
                minSize: tamMinimo,
                maxSize: tamMaximo,
                bufferParam: bufferParam,
                sizeBuffer: tamBuffer,
                continueCommand: continua
            }

            if (cancelledByUser) paramsContinua.continueCommand = STOP_TEF_COMMAND

            retornoContinua = await requesModule.curlRequest(EVENT_CONTINUA_TRANSACAO, paramsContinua)
            
            let bufferConverted = JSON.parse(retornoContinua.data.result).bufferConverted
            let retornoContinuaResult = Util.deepCopy(JSON.parse(retornoContinua.data.result))
            retornoContinuaResult.bufferConverted = bufferConverted
            
            // Conferirse esta na codificação correta
            //let bufferMessage = continuaTransacaoTefOBJ.encodeUtf8(retornoContinuaResult)
            //bufferMessage = bufferMessage.trim()
            retornoContinua = retornoContinuaResult
            //retornoContinua['bufferConverted'] = bufferMessage

            if (retornoContinua['retornoDll'] === 10000) {
                let result = continuaTransacaoTefOBJ.retornoContinuaTrad(retornoContinua, CDFILIAL = null, CDCAIXA = null, DATECANCEL, DOCNUMBER, valorTotal)
                comando = retornoContinua['comando']
                tipoCampo = retornoContinua['tipoCampo']
                tamMinimo = retornoContinua['tamMinimo']
                tamMaximo = retornoContinua['tamMaximo']
                tamBuffer = (result && result.tamBuffer) || 300
                bufferParam = ''

                if (result !== undefined) {
                    if (result.sendBuffer) {
                        bufferParam = result.bufferMessage
                    }

                    // Previne os erros qua causam loop infinito na rotina de cancelamento de TEF
                    if (result.error && result.message) {
                        let invalidDate = result.message === 'Data Invalida'
                        let invalidNsu = result.message === 'Documento Invalido'
                        let invalidParcel = result.message === 'Numero de Parcelas Invalido'
                        if (invalidDate || invalidNsu || invalidParcel) {
                            Util.showInfoModal(result.message)
                            break
                        }
                    }
                }

                if (comando === 21) {
                    if (isDigitalWalletPayment) {
                        // Nas operações de pagamento com carteira digital / PIX, pergunta ao usuário qual carteira deseja utilizar
                        eventModule.$emit('showWalletOptions')
                        bufferParam = PaymentController.getDigitalWallet()
                    } else if (!isCancelTefOperation) {
                        // Nas operações de pagamento com cartão, sempre passa a opção 1
                        bufferParam = '1'
                    } else {
                        // Operações de cancelamento de TEF
                        if (iterationCounter === 0) {
                            // Na primeira ocorrência do comando 21, é preciso verificar dinamicamente se a opção é
                            // 1 (DEBITO), 2 (CREDITO) ou 10 (PIX / CARTEIRA DIGITAL)
                            iterationCounter++
                            bufferParam = saleType
                        } else {
                            // A partir da segunda iteração, é preciso abrir o menu para o usuário escolher uma opção
                            eventModule.$emit('showCancelWalletOptions')
                            bufferParam = PaymentController.getDigitalWallet()
                        }
                    }
                }

                if (comando === 20 && isCancelTefOperation) {
                    // Por mais absurdo que pareça, de acordo com a doc do SiTef, 0 significa confirma e 1 signfica cancela
                    bufferParam = '0'
                }

                continua = retornoContinua['continua']
            } else if (retornoContinua['retornoDll'] !== 0) {
                return {
                    error: true,
                    retornoDll: retornoContinua.retornoDll,
                    message: errorMessage
                }
            }
        } while (retornoContinua['retornoDll'] === 10000)
        if (retornoContinua['retornoDll'] === 0) {
            return {
                'error': false,
                'finishedTransaction': true
            }
        } else {
            return continuaTransacaoTefOBJ.retornoDll(retornoContinua)
        }
    } catch (error) {
        console.log(error)
    }
}

const stopTransactionTef = () => {
    cancelledByUser = true
}

export default {
    continuaTransacaoTef,
    stopTransactionTef
}
