import requestModule from './requestModule.js'
import foodMenuController from '../controller/foodMenuController.js'
import moment from 'moment'
import notifier from './notifier.js'
import logger from './logger.js'

// Ignorar erro abaixo, o visual code nao conhece o assert =/
import config from './../../config.json' assert { type: "json" }

let getcachedfoodmenu = async function (reqbody, subItemPreSelectedMandatoryList = {}, force = false) {
    if (!force) {
        try {
            return foodMenuController.getCachedMenuFile(reqbody.posCode)
        } catch (error) {
            logger.log('Não havia cardápio pronto, buscando cardapio')
        }
    }

    let startTime = moment()
    logger.log(`Buscando Cardapio na odhen-interface`)
    let cardapio = await requestModule.post('v3/menu/getByPosExFull', reqbody).catch(e => logger.log(e))
    var duration = moment.duration(moment().diff(startTime));
    logger.log(`Requisição do cardápio para ${reqbody.posCode} durou ${duration.asSeconds()}s`)
    
    let cardGen = await foodMenuController.manageFoodMenu(cardapio, reqbody, subItemPreSelectedMandatoryList)

    return force ? cardGen : cardGen.data
}

let cacheallsst = async (cacheCardEarlier = false) => {
    logger.log('Iniciando cacheamento de todos os terminais - ' + moment().format('HH:mm:ss'))
    let payload = {
        'subsidiaryCode': config.filialCodigo,
        'storeCode': config.lojaCodigo
    }
    let ssts = await requestModule.post('menu/getPosTefData', payload).then(e => e.data.posTefData.filter(c => ['APC', 'TAA'].includes(c.posType))).catch(er=> logger.log(er, true))
    
    if (!ssts || ssts.length === 0) {
        logger.log('=============================================================================')
        logger.log('Verifique se as configurações estão corretas e se a interface está acessível e reinicie o serviço de menuCache')
        logger.log('=============================================================================')
        return false
     }
    notifier.sendNotification('TAAs encontrados para cachear o cardápio '+ ssts.length)
    let subItemPreSelectedMandatoryListCached = {}
    for (let i = 0; i < ssts.length; i++) {
        const caixa = ssts[i];
        payload = {
            'subsidiaryCode': config.filialCodigo,
            'storeCode': config.lojaCodigo,
            'posCode': caixa.posCode,
            'consumerType': null,
            'hour': cacheCardEarlier ? moment().add(15, 'minutes').format('HH:mm') : moment().format('HH:mm')
        }
        let cardGen = await getcachedfoodmenu(payload, subItemPreSelectedMandatoryListCached, true)
        subItemPreSelectedMandatoryListCached = cardGen.subItemPreSelectedMandatoryList
    }
    notifier.sendNotification('Todos cardápios dos ' + ssts.length + ' terminais foram cacheados no servidor.')
}

export default {
    purgecache: () => foodMenuController.purgeCache(),
    purgecachebyname: (reqbody) => foodMenuController.purgeCacheByName(reqbody.posCode),
    getcachedfoodmenu,
    cacheallsst,
}
